#!/usr/bin/env python
#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import unittest

from _test_client_base import RpclibClientTestBase
from rpclib.client.http import Client
from rpclib.test.interop.server.soap_http_basic import soap_application
from rpclib.util.etreeconv import root_dict_to_etree

class TestRpclibHttpClient(RpclibClientTestBase, unittest.TestCase):
    def setUp(self):
        self.client = Client('http://localhost:9753/', soap_application)
        self.ns = "rpclib.test.interop.server._service"

    def test_any(self):
        val = root_dict_to_etree(self._get_xml_test_val())
        ret = self.client.service.echo_any(val)

        self.assertEquals(ret, val)

if __name__ == '__main__':
    unittest.main()




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from _base import base_to_parent_element
from _base import nillable_element
from _base import nillable_value

@nillable_value
def enum_to_parent_element(prot, cls, value, tns, parent_elt, name='retval'):
    if name is None:
        name = cls.get_type_name()

    base_to_parent_element(prot, cls, str(value), tns, parent_elt, name)

@nillable_element
def enum_from_element(prot, cls, element):
    return getattr(cls, element.text)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""
The rpclib.model.table module is EXPERIMENTAL. Please report any issues you
might find.
"""

import logging
logger = logging.getLogger(__name__)

import sqlalchemy
from sqlalchemy import Column

from sqlalchemy.ext.declarative import DeclarativeMeta

from sqlalchemy.dialects.postgresql import UUID

from rpclib.model.complex import TypeInfo
from rpclib.model.complex import ComplexModelBase
from rpclib.model.complex import ComplexModelMeta
from rpclib.model import primitive
from rpclib.model import complex

_type_map = {
    sqlalchemy.Text: primitive.String,
    sqlalchemy.String: primitive.String,
    sqlalchemy.Unicode: primitive.String,
    sqlalchemy.UnicodeText: primitive.String,

    sqlalchemy.Float: primitive.Float,
    sqlalchemy.Numeric: primitive.Decimal,
    sqlalchemy.Integer: primitive.Integer,
    sqlalchemy.SmallInteger: primitive.Integer,

    sqlalchemy.Boolean: primitive.Boolean,
    sqlalchemy.DateTime: primitive.DateTime,
    sqlalchemy.orm.relation: complex.Array,
    UUID: primitive.String
}

def _process_item(v):
    if v.type in _type_map:
        rpc_type = _type_map[v.type]
    elif type(v.type) in _type_map:
        rpc_type = _type_map[type(v.type)]
    else:
        raise Exception("soap_type was not found. maybe _type_map needs a new "
                        "entry. %r" % v)

    return rpc_type

class TableSerializerMeta(DeclarativeMeta, ComplexModelMeta):
    def __new__(cls, cls_name, cls_bases, cls_dict):
        if cls_dict.get("__type_name__", None) is None:
            cls_dict["__type_name__"] = cls_name

        if cls_dict.get("_type_info", None) is None:
            cls_dict["_type_info"] = _type_info = TypeInfo()

            # mixin inheritance
            for b in cls_bases:
                for k,v in vars(b).items():
                    if isinstance(v, Column):
                        _type_info[k] = _process_item(v)

            # same table inheritance
            for b in cls_bases:
                table = getattr(b, '__table__', None)

                if not (table is None):
                    for c in table.c:
                        _type_info[c.name] = _process_item(c)

            # include from table
            table = cls_dict.get('__table__', None)
            if not (table is None):
                for c in table.c:
                    _type_info[c.name] = _process_item(c)

            # own attributes
            for k, v in cls_dict.items():
                if (not k.startswith('__')) and isinstance(v, Column):
                    _type_info[k] = _process_item(v)

        return DeclarativeMeta.__new__(cls, cls_name, cls_bases, cls_dict)

class TableSerializer(ComplexModelBase):
    __metaclass__ = TableSerializerMeta
    _decl_class_registry = {}

    @classmethod
    def customize(cls, **kwargs):
        cls_name, cls_bases, cls_dict = ComplexModelBase._s_customize(
                                                                  cls, **kwargs)

        retval = ComplexModelMeta.__new__(ComplexModelMeta, cls_name,
                                                            cls_bases, cls_dict)

        return retval




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from lxml import etree

from rpclib.util.etreeconv import etree_to_dict
from rpclib.util.etreeconv import dict_to_etree

from _base import base_to_parent_element
from _base import nillable_value
from _base import nillable_element

@nillable_element
def xml_from_element(prot, cls, element):
    children = element.getchildren()
    retval = None

    if children:
        retval = element.getchildren()[0]

    return retval

@nillable_value
def xml_to_parent_element(prot, cls, value, tns, parent_elt, name='retval'):
    if isinstance(value, str) or isinstance(value, unicode):
        value = etree.fromstring(value)

    e = etree.SubElement(parent_elt, '{%s}%s' % (tns,name))
    e.append(value)

@nillable_value
def dict_to_parent_element(prot, cls, value, tns, parent_elt, name='retval'):
    e = etree.SubElement(parent_elt, '{%s}%s' % (tns,name))
    dict_to_etree(e, value)

@nillable_element
def dict_from_element(prot, cls, element):
    children = element.getchildren()
    if children:
        return etree_to_dict(element)

    return None

@nillable_element
def string_from_element(prot, cls, element):
    return element.text or u""

@nillable_value
def string_to_parent_element(prot, cls, value, tns, parent_elt, name='retval'):
    elt = etree.SubElement(parent_elt, "{%s}%s" % (tns, name))
    elt.text = value

@nillable_element
def decimal_from_element(prot, cls, element):
    return element.text or u""

@nillable_value
def decimal_to_parent_element(prot, cls, value, tns, parent_elt, name='retval'):
    elt = etree.SubElement(parent_elt, "{%s}%s" % (tns, name))
    elt.text = value

@nillable_value
def duration_to_parent_element(cls, value, tns, parent_elt, name='retval'):
    base_to_parent_element(cls.from_string(value), tns, parent_elt, name)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from lxml import etree

import rpclib.const.xml_ns
_ns_xsd = rpclib.const.xml_ns.xsd

def enum_add(interface, cls):
    if not interface.has_class(cls):
        simple_type = etree.Element('{%s}simpleType' % _ns_xsd)
        simple_type.set('name', cls.get_type_name())

        restriction = etree.SubElement(simple_type,
                                            '{%s}restriction' % _ns_xsd)
        restriction.set('base', '%s:string' %
                                interface.get_namespace_prefix(_ns_xsd))

        for v in cls.__values__:
            enumeration = etree.SubElement(restriction,
                                            '{%s}enumeration' % _ns_xsd)
            enumeration.set('value', v)

        interface.add_simple_type(cls, simple_type)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import re
import math
import pytz
import decimal

import rpclib.const.xml_ns
import cPickle as pickle

from collections import deque

from datetime import date
from datetime import datetime
from datetime import timedelta

from lxml import etree
from pytz import FixedOffset

from rpclib.model import SimpleModel
from rpclib.model import nillable_string

string_encoding = 'utf8'

_date_pattern = r'(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})'
_time_pattern = r'(?P<hr>\d{2}):(?P<min>\d{2}):(?P<sec>\d{2})(?P<sec_frac>\.\d+)?'
_offset_pattern = r'(?P<tz_hr>[+-]\d{2}):(?P<tz_min>\d{2})'
_datetime_pattern = _date_pattern + '[T ]' + _time_pattern

_local_re = re.compile(_datetime_pattern)
_utc_re = re.compile(_datetime_pattern + 'Z')
_offset_re = re.compile(_datetime_pattern + _offset_pattern)
_date_re = re.compile(_date_pattern)
_duration_re = re.compile(
        r'(?P<sign>-?)'
        r'P'
        r'(?:(?P<years>\d+)Y)?'
        r'(?:(?P<months>\d+)M)?'
        r'(?:(?P<days>\d+)D)?'
        r'(?:T(?:(?P<hours>\d+)H)?'
        r'(?:(?P<minutes>\d+)M)?'
        r'(?:(?P<seconds>\d+(.\d+)?)S)?)?'
    )

_ns_xs = rpclib.const.xml_ns.xsd
_ns_xsi = rpclib.const.xml_ns.xsi

class AnyXml(SimpleModel):
    __type_name__ = 'anyType'

    @classmethod
    @nillable_string
    def to_string(cls, value):
        return etree.tostring(value)

    @classmethod
    @nillable_string
    def from_string(cls, string):
        return etree.fromstring(string)

class AnyDict(SimpleModel):
    __type_name__ = 'anyType'

    @classmethod
    @nillable_string
    def to_string(cls, value):
        return pickle.dumps(value)

    @classmethod
    @nillable_string
    def from_string(cls, string):
        return pickle.loads(string)

class String(SimpleModel):
    __type_name__ = 'string'

    class Attributes(SimpleModel.Attributes):
        min_len = 0
        max_len = "unbounded"
        pattern = None

    def __new__(cls, *args, **kwargs):
        assert len(args) <= 1

        if len(args) == 1:
            kwargs['max_len'] = args[0]

        retval = SimpleModel.__new__(cls,  ** kwargs)

        return retval

    @classmethod
    @nillable_string
    def from_string(cls, value):
        return value

    @staticmethod
    def is_default(cls):
        return (    SimpleModel.is_default(cls)
                and cls.Attributes.min_len == String.Attributes.min_len
                and cls.Attributes.max_len == String.Attributes.max_len
                and cls.Attributes.pattern == String.Attributes.pattern)

class AnyUri(String):
    __type_name__ = 'anyURI'

class Decimal(SimpleModel):
    __type_name__ = 'decimal'

    class Attributes(SimpleModel.Attributes):
        gt = None # minExclusive
        ge = None # minInclusive
        lt = None # maxExclusive
        le = None # maxInclusive

    @staticmethod
    def is_default(cls):
        return (    SimpleModel.is_default(cls)
                and cls.Attributes.gt == Decimal.Attributes.gt
                and cls.Attributes.ge == Decimal.Attributes.ge
                and cls.Attributes.lt == Decimal.Attributes.lt
                and cls.Attributes.le == Decimal.Attributes.le)

    @classmethod
    @nillable_string
    def to_string(cls, value):
        decimal.Decimal(value)

        return str(value)

    @classmethod
    @nillable_string
    def from_string(cls, string):
        return decimal.Decimal(string)

class Int(Decimal):
    __type_name__ = 'int'

    @classmethod
    @nillable_string
    def to_string(cls, value):
        int(value)
        return str(value)

    @classmethod
    @nillable_string
    def from_string(cls, string):
        return int(string)

class Integer(Decimal):
    __type_name__ = 'integer'

    @classmethod
    @nillable_string
    def to_string(cls, value):
        try:
            int(value)
        except:
            long(value)

        return str(value)

    @classmethod
    @nillable_string
    def from_string(cls, string):
        try:
            return int(string)
        except:
            return long(string)

class UnsignedInteger(Integer):
    __type_name__ = 'unsignedLong'
    __length__ = None
    @classmethod
    @nillable_string
    def to_string(cls, value):
        assert (cls.__length__ is None) or (0 <= value < 2**cls.__length__)

        return str(value)

    @classmethod
    @nillable_string
    def from_string(cls, string):
        try:
            retval = int(string)
        except:
            retval = long(string)

        assert (cls.__length__ is None) or (0 <= retval < 2**cls.__length__)

        return retval

class UnsignedInteger64(UnsignedInteger):
    __type_name__ = 'unsignedLong'
    __length__ = 64

class UnsignedInteger32(UnsignedInteger):
    __type_name__ = 'unsignedLong'
    __length__ = 32

class UnsignedInteger16(Integer):
    __type_name__ = 'unsignedShort'
    __length__ = 16

class UnsignedInteger8(Integer):
    __type_name__ = 'unsignedByte'
    __length__ = 8

class Date(SimpleModel):
    __type_name__ = 'date'

    @classmethod
    @nillable_string
    def to_string(cls, value):
        return value.isoformat()

    @classmethod
    @nillable_string
    def from_string(cls, string):
        """expect ISO formatted dates"""
        def parse_date(date_match):
            fields = date_match.groupdict(0)
            year, month, day = [int(fields[x]) for x in
                ("year", "month", "day")]
            return date(year, month, day)

        match = _date_re.match(string)
        if not match:
            raise Exception("Date [%s] not in known format" % string)

        return parse_date(match)

class DateTime(SimpleModel):
    __type_name__ = 'dateTime'

    @classmethod
    @nillable_string
    def to_string(cls, value):
        return value.isoformat('T')

    @classmethod
    @nillable_string
    def from_string(cls, string):
        """expect ISO formatted dates"""
        def parse_date(date_match, tz=None):
            fields = date_match.groupdict(0)
            year, month, day, hour, min, sec = [int(fields[x]) for x in
                ("year", "month", "day", "hr", "min", "sec")]

            # use of decimal module here (rather than float) might be better
            # here, if willing to require python 2.4 or higher
            microsec = int(float(fields.get("sec_frac", 0)) * 10 ** 6)

            return datetime(year,month,day, hour,min,sec, microsec, tz)

        match = _utc_re.match(string)
        if match:
            return parse_date(match, tz=pytz.utc)

        match = _offset_re.match(string)
        if match:
            tz_hr, tz_min = [int(match.group(x)) for x in "tz_hr", "tz_min"]
            return parse_date(match, tz=FixedOffset(tz_hr * 60 + tz_min, {}))

        match = _local_re.match(string)
        if not match:
            raise Exception("DateTime [%s] not in known format" % string)

        return parse_date(match)

# this object tries to follow ISO 8601 standard.
class Duration(SimpleModel):
    __type_name__ = 'duration'

    @classmethod
    @nillable_string
    def from_string(cls, string):
        duration = _duration_re.match(string).groupdict(0)

        days = int(duration['days'])
        days += int(duration['months']) * 30
        days += int(duration['years']) * 365
        hours = int(duration['hours'])
        minutes = int(duration['minutes'])
        seconds = float(duration['seconds'])
        f,i = math.modf(seconds)
        seconds = i
        microseconds = int(1e6 * f)

        delta = timedelta(days=days, hours=hours, minutes=minutes,
                                    seconds=seconds, microseconds=microseconds)

        if duration['sign'] == "-":
            delta *= -1

        return delta

    @classmethod
    def to_string(cls, value):
        if value.days < 0:
            value = -value
            negative = True
        else:
            negative = False

        seconds = value.seconds % 60
        minutes = value.seconds / 60
        hours = minutes / 60
        minutes = minutes % 60
        seconds = float(seconds) + value.microseconds / 1e6

        retval = deque()
        if negative:
            retval.append("-")

        retval = ['P']
        if value.days > 0:
            retval.extend([
                    "%iD" % value.days,
                ])

        if hours > 0 and minutes > 0 and seconds > 0:
            retval.extend([
                    "T",
                    "%iH" % hours,
                    "%iM" % minutes,
                    "%fS" % seconds,
                ])

        else:
            retval.extend([
                    "0S",
                ])

        return ''.join(retval)

class Double(SimpleModel):
    __type_name__ = 'double'

    @classmethod
    @nillable_string
    def to_string(cls, value):
        return repr(value)

    @classmethod
    @nillable_string
    def from_string(cls, string):
        return float(string)

class Float(Double):
    __type_name__ = 'float'

class Boolean(SimpleModel):
    __type_name__ = 'boolean'

    @classmethod
    @nillable_string
    def to_string(cls, value):
        return str(bool(value)).lower()

    @classmethod
    @nillable_string
    def from_string(cls, string):
        return (string.lower() in ['true', '1'])

# a class that is really a namespace
class Mandatory(object):
    String = String(min_occurs=1, nillable=False, min_len=1)
    Integer = Integer(min_occurs=1, nillable=False)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from lxml import etree

from rpclib.const import xml_ns as namespace
from rpclib.model import ModelBase
from rpclib.model.complex import XMLAttribute

def complex_add(interface, cls):
    if cls.get_type_name() is ModelBase.Empty:
        (child, ) = cls._type_info.values()
        cls.__type_name__ = '%sArray' % child.get_type_name()

    if not interface.has_class(cls):
        extends = getattr(cls, '__extends__', None)
        if not (extends is None):
            interface.add(extends)

        complex_type = etree.Element("{%s}complexType" % namespace.xsd)
        complex_type.set('name', cls.get_type_name())

        sequence_parent = complex_type
        if not (extends is None):
            complex_content = etree.SubElement(complex_type,
                                       "{%s}complexContent" % namespace.xsd)
            extension = etree.SubElement(complex_content,
                                       "{%s}extension" % namespace.xsd)
            extension.set('base', extends.get_type_name_ns(interface))
            sequence_parent = extension

        sequence = etree.SubElement(sequence_parent, '{%s}sequence' %
                                                                 namespace.xsd)

        for k, v in cls._type_info.items():
            if isinstance(v, XMLAttribute):
                attribute = etree.SubElement(complex_type,
                                            '{%s}attribute' % namespace.xsd)
                v.describe(k, attribute)
                continue

            if v != cls:
                interface.add(v)

            member = etree.SubElement(sequence, '{%s}element' % namespace.xsd)
            member.set('name', k)
            member.set('type', v.get_type_name_ns(interface))

            if v.Attributes.min_occurs != 1: # 1 is the xml schema default
                member.set('minOccurs', str(v.Attributes.min_occurs))
            if v.Attributes.max_occurs != 1: # 1 is the xml schema default
                member.set('maxOccurs', str(v.Attributes.max_occurs))

            if bool(v.Attributes.nillable) == True:
                member.set('nillable', 'true')
            #else:
            #    member.set('nillable', 'false')

            if v.Annotations.doc != '':
                annotation = etree.SubElement(member, "{%s}annotation",
                                                              namespace.xsd)
                doc = etree.SubElement(annotation, "{%s}documentation",
                                                              namespace.xsd)
                doc.text = v.Annotations.doc

        interface.add_complex_type(cls, complex_type)

        # simple node
        element = etree.Element('{%s}element' % namespace.xsd)
        element.set('name', cls.get_type_name())
        element.set('type', cls.get_type_name_ns(interface))

        interface.add_element(cls, element)

def alias_add(interface, cls):
    if not interface.has_class(cls._target):
        interface.add(cls._target)
    element = etree.Element('{%s}element' % namespace.xsd)
    element.set('name', cls.get_type_name())
    element.set('type', cls._target.get_type_name_ns(interface.app))

    interface.add_element(cls, element)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

import rpclib.const.xml_ns

_ns_xsi = rpclib.const.xml_ns.xsi
_ns_xsd = rpclib.const.xml_ns.xsd

from rpclib._base import EventManager
from rpclib.model.fault import Fault

class ValidationError(Fault):
    pass

class ProtocolBase(object):
    allowed_http_verbs = ['GET','POST']
    mime_type = 'application/octet-stream'

    def __init__(self, app=None):
        self.__app = None

        self.set_app(app)
        self.event_manager = EventManager(self)

    @property
    def app(self):
        return self.__app

    def set_app(self, value):
        assert self.__app is None, "One protocol instance should belong to one " \
                                   "application instance."
        self.__app = value

    def create_in_document(self, ctx, in_string_encoding=None):
        """Uses ctx.in_string to set ctx.in_document"""

    def decompose_incoming_envelope(self, ctx):
        """Sets the ctx.in_body_doc, ctx.in_header_doc and ctx.service
        properties of the ctx object, if applicable.
        """

    def deserialize(self, ctx):
        """Takes a MethodContext instance and a string containing ONE document
        instance in the ctx.in_string attribute.

        Returns the corresponding native python object in the ctx.in_object
        attribute.
        """

    def serialize(self, ctx):
        """Takes a MethodContext instance and the object to be serialied in the
        ctx.out_object attribute.

        Returns the corresponding document structure in the ctx.out_document
        attribute.
        """

    def create_out_string(self, ctx, out_string_encoding=None):
        """Uses ctx.out_string to set ctx.out_document"""

    def validate(self, payload):
        """Method to be overriden to perform any sort of custom input
        validation.
        """

    def set_method_descriptor(self, ctx):
        """Method to be overriden to perform any sort of custom matching between
        the method_request_string and the methods.
        """

        name = ctx.method_request_string
        if not name.startswith("{"):
            name = '{%s}%s' % (self.app.interface.get_tns(), name)

        ctx.service_class = self.app.interface.service_mapping.get(name, None)
        if ctx.service_class is None:
            logger.debug(self.app.interface.service_mapping.keys())
            raise Exception('Method %r not bound to a service class.' % name)

        ctx.descriptor = ctx.app.interface.method_mapping.get(name, None)
        if ctx.descriptor is None:
            logger.debug(ctx.app.interface.method_mapping.keys())
            raise Exception('Method %r not found.' % name)



#!/usr/bin/env python
#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('rpclib.wsgi')
logger.setLevel(logging.DEBUG)

from twisted.python import log

from rpclib.test.interop.server.soap_http_basic import soap_application

from rpclib.util.wsgi_wrapper import run_twisted
from rpclib.server.wsgi import WsgiApplication

port = 9754
url = 'app'

def main(argv):
    observer = log.PythonLoggingObserver('twisted')
    log.startLoggingWithObserver(observer.emit, setStdout=False)

    wsgi_application = WsgiApplication(soap_application)

    return run_twisted( [ (wsgi_application, url) ], port )

if __name__ == '__main__':
    import sys
    sys.exit(main(sys.argv))



#!/usr/bin/env python
#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import base64
import os
import shutil
import unittest
from tempfile import mkstemp
from lxml import etree

from rpclib.model.binary import Attachment
import rpclib.const.xml_ns

ns_xsd = rpclib.const.xml_ns.xsd
ns_test = 'test_namespace'

class TestBinary(unittest.TestCase):
    def setUp(self):
        os.mkdir('binaryDir')

        fd, self.tmpfile = mkstemp('', '', 'binaryDir')
        os.close(fd)
        f = open(self.tmpfile, 'w')
        for i in range(0, 1000):
            f.write('All work and no play makes jack a dull boy\r\n')
        f.flush()
        f.close()

    def tearDown(self):
        shutil.rmtree('binaryDir')

    def test_to_parent_element_data(self):
        f = open(self.tmpfile)
        data = f.read()
        f.close()
        a = Attachment()
        a.data = data
        element = etree.Element('test')
        Attachment.to_parent_element(a, ns_test, element)
        element = element[0]
        encoded_data = base64.encodestring(data)
        self.assertNotEquals(element.text, None)
        self.assertEquals(element.text, encoded_data)

    def test_to_parent_element_file(self):
        a = Attachment()
        a.file_name = self.tmpfile
        f = open(self.tmpfile, 'rb')
        data = f.read()
        f.close()
        element = etree.Element('test')
        Attachment.to_parent_element(a, ns_test, element)
        element = element[0]
        encoded_data = base64.encodestring(data)
        self.assertNotEquals(element.text, None)
        self.assertEquals(element.text, encoded_data)

    def test_to_from_xml_file(self):
        a = Attachment()
        a.file_name = self.tmpfile
        element = etree.Element('test')
        Attachment.to_parent_element(a, ns_test, element)
        element = element[0]

        data = Attachment.from_xml(element).data

        f = open(self.tmpfile, 'rb')
        fdata = f.read()
        f.close()

        self.assertEquals(data, fdata)

    def test_exception(self):
        try:
            Attachment.to_parent_element(Attachment(), ns_test)
        except:
            self.assertTrue(True)
        else:
            self.assertFalse(True)

    def test_from_xml(self):
        f = open(self.tmpfile)
        data = f.read()
        f.close()

        a = Attachment()
        a.data = data
        element = etree.Element('test')
        Attachment.to_parent_element(a, ns_test, element)
        element = element[0]
        a2 = Attachment.from_xml(element)

        self.assertEquals(data, a2.data)

    def test_add_to_schema(self):
        schema = {}
        Attachment.add_to_schema(schema)
        self.assertEquals(0, len(schema.keys()))

    def test_get_datatype(self):
        dt = Attachment.get_type_name()
        self.assertEquals('base64Binary', dt)

        dt = Attachment.get_namespace()
        assert dt == ns_xsd

if __name__ == '__main__':
    unittest.main()




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

from rpclib._base import EventManager
from rpclib.util.oset import oset

class ServiceBaseMeta(type):
    def __init__(self, cls_name, cls_bases, cls_dict):
        super(ServiceBaseMeta, self).__init__(cls_name, cls_bases, cls_dict)

        self.public_methods = {}
        self.event_manager = EventManager(self,
                                      self.__get_base_event_handlers(cls_bases))

        for k, v in cls_dict.iteritems():
            if hasattr(v, '_is_rpc'):
                descriptor = v(_default_function_name=k)
                self.public_methods[k] = descriptor
                setattr(self, k, staticmethod(descriptor.function))
                descriptor.function = getattr(self, k)

    def __get_base_event_handlers(self, cls_bases):
        handlers = {}

        for base in cls_bases:
            evmgr = getattr(base,'event_manager',None)
            if evmgr is None:
                continue

            for k,v in evmgr.handlers.items():
                handler=handlers.get(k,oset())
                for h in v:
                    handler.add(h)
                handlers[k]=handler

        return handlers

class ServiceBase(object):
    '''This class serves as the base for all service definitions.  Subclasses of
    this class will use the rpc decorator to flag methods to be exposed via soap.

    It is a natural abstract base class, because it's of no use without any
    method definitions, hence the 'Base' suffix in the name.
    '''

    __metaclass__ = ServiceBaseMeta

    __tns__ = None
    __in_header__ = None
    __out_header__ = None
    __service_name__ = None
    __port_types__ = ()

    @classmethod
    def get_service_class_name(cls):
        return cls.__name__

    @classmethod
    def get_service_name(cls):
        return cls.__service_name__

    @classmethod
    def get_port_types(cls):
        return cls.__port_types__

    @classmethod
    def get_tns(cls):
        if not (cls.__tns__ is None):
            return cls.__tns__

        retval = cls.__module__
        if cls.__module__ == '__main__':
            service_name = cls.get_service_class_name().split('.')[-1]
            retval = '.'.join((service_name, service_name))

        return retval

    @classmethod
    def _has_callbacks(cls):
        '''Determines if this object has callback methods or not.'''

        for method in cls.public_methods.values():
            if method.is_callback:
                return True

        return False

    @classmethod
    def call_wrapper(cls, ctx):
        '''Called in place of the original method call.

        @param the original method call
        @param the arguments to the call
        '''
        if ctx.descriptor.no_ctx:
            return ctx.descriptor.function(*ctx.in_object)
        else:
            return ctx.descriptor.function(ctx, *ctx.in_object)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

import warnings
from rpclib.util.odict import odict

import rpclib.const.xml_ns
_ns_xsd = rpclib.const.xml_ns.xsd

class SchemaInfo(object):
    def __init__(self):
        self.elements = odict()
        self.types = odict()

class InterfaceBase(object):
    def __init__(self, app=None):
        self.__ns_counter = 0

        self.service_mapping = {}
        self.method_mapping = {}

        self.url = None

        self.__app = None
        self.set_app(app)

    def set_app(self, value):
        assert self.__app is None, "One interface instance should belong to one " \
                                   "application instance."

        self.__app = value
        self.populate_interface()

    @property
    def app(self):
        return self.__app

    @property
    def services(self):
        if self.__app:
            return self.__app.services
        return []

    def reset_interface(self):
        self.namespaces = odict()
        self.classes = {}
        self.imports = {}

    def has_class(self, cls):
        ns_prefix = cls.get_namespace_prefix(self)
        type_name = cls.get_type_name()

        return ((ns_prefix in self.namespaces) and
                           (type_name in self.namespaces[ns_prefix].types))

    def get_schema_info(self, prefix):
        if prefix in self.namespaces:
            schema = self.namespaces[prefix]
        else:
            schema = self.namespaces[prefix] = SchemaInfo()

        return schema

    def get_class(self, key):
        return self.classes[key]

    def get_class_instance(self, key):
        return self.classes[key]()

    def get_name(self):
        """Returns service name that is seen in the name attribute of the
        definitions tag.

        Not meant to be overridden.
        """
        if self.app:
            return self.app.name

    def get_tns(self):
        """Returns default namespace that is seen in the targetNamespace
        attribute of the definitions tag.

        Not meant to be overridden.
        """
        if self.app:
            return self.app.tns

    def populate_interface(self, types=None):
        # FIXME: should also somehow freeze child classes' _type_info
        #        dictionaries.

        self.reset_interface()

        # populate types
        for s in self.services:
            logger.debug("populating '%s.%s' types..." % (s.__module__, s.__name__))
            for method in s.public_methods.values():
                if method.in_header is None:
                    method.in_header = s.__in_header__
                if method.out_header is None:
                    method.out_header = s.__out_header__

                if not (method.in_header is None):
                    if isinstance(method.in_header, (list, tuple)):
                        in_headers = method.in_header
                    else:
                        in_headers = (method.in_header,)
                    for in_header in in_headers:
                        in_header.resolve_namespace(in_header, self.get_tns())
                        self.add(in_header)

                if not (method.out_header is None):
                    if isinstance(method.out_header, (list,tuple)):
                        out_headers = method.out_header
                    else:
                        out_headers = (method.out_header,)

                    for out_header in out_headers:
                        out_header.resolve_namespace(out_header, self.get_tns())
                        self.add(out_header)

                method.in_message.resolve_namespace(method.in_message,
                                                                 self.get_tns())
                self.add(method.in_message)

                method.out_message.resolve_namespace(method.out_message,
                                                                 self.get_tns())
                self.add(method.out_message)

                for fault in method.faults:
                    fault.resolve_namespace(fault, self.get_tns())
                    self.add(fault)

        # populate call routes
        for s in self.services:
            s.__tns__ = self.get_tns()
            logger.debug("populating '%s.%s' methods..." % (s.__module__, s.__name__))
            for method in s.public_methods.values():
                o = self.service_mapping.get(method.key)
                if not (o is None):
                    raise Exception("\nThe message %r defined in both '%s.%s'"
                                                                " and '%s.%s'"
                      % (method.key, s.__module__, s.__name__,
                                          o.__module__, o.__name__,
                        ))

                else:
                    logger.debug('\tadding method %r to match %r tag.' %
                                                      (method.name, method.key))
                    self.service_mapping[method.key] = s
                    self.method_mapping[method.key] = method

    tns = property(get_tns)

    def get_namespace_prefix(self, ns):
        """Returns the namespace prefix for the given namespace. Creates a new
        one automatically if it doesn't exist.

        Not meant to be overridden.
        """

        if not (isinstance(ns, str) or isinstance(ns, unicode)):
            raise TypeError(ns)

        if ns == "__main__":
            warnings.warn("Namespace is '__main__'", Warning )

        if not (ns in self.prefmap):
            pref = "s%d" % self.__ns_counter
            while pref in self.nsmap:
                self.__ns_counter += 1
                pref = "s%d" % self.__ns_counter

            self.prefmap[ns] = pref
            self.nsmap[pref] = ns

            self.__ns_counter += 1

        else:
            pref = self.prefmap[ns]

        return pref

    def add(self, cls):
        raise NotImplementedError('Extend and override.')




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

from rpclib.protocol import ProtocolBase
import urlparse

# this is not exactly rest, because it ignores http verbs.
class HttpRpc(ProtocolBase):
    def create_in_document(self, ctx, in_string_encoding=None):
        assert ctx.transport.type == 'wsgi', ("This protocol only works with "
                                              "the wsgi api.")

        logger.debug("PATH_INFO: %r" % ctx.transport.req_env['PATH_INFO'])
        logger.debug("QUERY_STRING: %r" % ctx.transport.req_env['QUERY_STRING'])

        ctx.method_request_string = '{%s}%s' % (self.app.interface.get_tns(),
                              ctx.transport.req_env['PATH_INFO'].split('/')[-1])
        logger.debug("\033[92mMethod name: %r\033[0m" % ctx.method_request_string)

        self.app.in_protocol.set_method_descriptor(ctx)

        ctx.in_header_doc = None
        ctx.in_body_doc = urlparse.parse_qs(ctx.transport.req_env['QUERY_STRING'])

        logger.debug(repr(ctx.in_body_doc))

        return ctx.in_body_doc

    def deserialize(self, ctx):
        body_class = ctx.descriptor.in_message
        if ctx.in_body_doc is not None and len(ctx.in_body_doc) > 0:
            ctx.in_object = body_class.from_dict(ctx.in_body_doc)
        else:
            ctx.in_object = [None] * len(body_class._type_info)

        self.event_manager.fire_event('deserialize', ctx)

    def serialize(self, ctx):
        result_message_class = ctx.descriptor.out_message
        result_message = result_message_class()

        # assign raw result to its wrapper, result_message
        out_type_info = result_message_class._type_info
        if len(out_type_info) > 0:
             if len(out_type_info) == 1:
                 attr_name = result_message_class._type_info.keys()[0]
                 setattr(result_message, attr_name, ctx.out_object)

             else:
                 for i in range(len(out_type_info)):
                     attr_name=result_message_class._type_info.keys()[i]
                     setattr(result_message, attr_name, ctx.out_object[i])

        wrapped_result = result_message_class.to_dict(result_message)

        ctx.out_document, = wrapped_result.itervalues()

        self.event_manager.fire_event('serialize', ctx)

    def create_out_string(self, ctx, out_string_encoding=None):
        ctx.out_string = ctx.out_document




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from lxml import etree

import rpclib.const.xml_ns

_ns_xsd = rpclib.const.xml_ns.xsd
_ns_soap_env = rpclib.const.xml_ns.soap_env

_pref_soap_env = rpclib.const.xml_ns.const_prefmap[_ns_soap_env]

def fault_to_parent_element(prot, cls, value, tns, parent_elt, name=None):
    element = etree.SubElement(parent_elt, "{%s}Fault" % _ns_soap_env)

    etree.SubElement(element, 'faultcode').text = value.faultcode
    etree.SubElement(element, 'faultstring').text = value.faultstring
    etree.SubElement(element, 'faultactor').text = value.faultactor
    if value.detail != None:
        etree.SubElement(element, 'detail').append(value.detail)

def fault_from_element(prot, cls, element):
    code = element.find('faultcode').text
    string = element.find('faultstring').text
    factor = element.find('faultactor').text
    detail = element.find('detail')

    return cls(faultcode=code, faultstring=string, faultactor=factor,
                                                                  detail=detail)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

from rpclib.model.fault import Fault
from rpclib._base import EventManager

class Application(object):
    transport = None

    def __init__(self, services, tns, interface, in_protocol, out_protocol,
                                                                    name=None):
        '''Constructor.

        @param An iterable of ServiceBase subclasses that define the exposed
               services.
        @param The targetNamespace attribute of the exposed service.
        @param The name attribute of the exposed service.
        '''

        self.services = services
        self.tns = tns
        self.name = name
        if self.name is None:
            self.name = self.__class__.__name__.split('.')[-1]

        self.interface = interface
        self.interface.set_app(self)

        self.in_protocol = in_protocol
        self.in_protocol.set_app(self)

        self.out_protocol = out_protocol
        self.out_protocol.set_app(self)

        self.__public_methods = {}
        self.__classes = {}

        self.event_manager = EventManager(self)
        self.error_handler = None

    def process_request(self, ctx):
        """Takes a MethodContext instance and the native request object.
        Returns the response to the request as a native python object.

        Not meant to be overridden.
        """

        try:
            # fire events
            self.event_manager.fire_event('method_call', ctx)
            ctx.service_class.event_manager.fire_event('method_call', ctx)

            # call the method
            ctx.out_object = self.call_wrapper(ctx)

            # fire events
            self.event_manager.fire_event('method_return_object', ctx)
            ctx.service_class.event_manager.fire_event(
                                                    'method_return_object', ctx)

        except Fault, e:
            logger.exception(e)

            ctx.out_error = e

            # fire events
            self.event_manager.fire_event('method_exception_object', ctx)
            if ctx.service_class != None:
                ctx.service_class.event_manager.fire_event(
                                                    'method_return_object', ctx)

        except Exception, e:
            logger.exception(e)

            ctx.out_error = Fault('Server', str(e))

            # fire events
            self.event_manager.fire_event('method_exception_object', ctx)
            if ctx.service_class != None:
                ctx.service_class.event_manager.fire_event(
                                                    'method_return_object', ctx)

    def call_wrapper(self, ctx):
        return ctx.service_class.call_wrapper(ctx)

    def _has_callbacks(self):
        return self.interface._has_callbacks()




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""An rpc server that uses http as transport, and wsgi as bridge api"""

# FIXME: this is maybe still too soap-centric.

import logging
logger = logging.getLogger(__name__)

import cgi

from rpclib._base import MethodContext
from rpclib.model.fault import Fault
from rpclib.protocol.soap.mime import apply_mtom
from rpclib.util import reconstruct_url
from rpclib.server import ServerBase


HTTP_500 = '500 Internal server error'
HTTP_200 = '200 OK'
HTTP_404 = '404 Method Not Found'
HTTP_405 = '405 Method Not Allowed'


def reconstruct_wsgi_request(http_env):
    """Reconstruct http payload using information in the http header"""

    input = http_env.get('wsgi.input')
    try:
        length = int(http_env.get("CONTENT_LENGTH"))
    except ValueError:
        length = 0

    # fyi, here's what the parse_header function returns:
    # >>> import cgi; cgi.parse_header("text/xml; charset=utf-8")
    # ('text/xml', {'charset': 'utf-8'})
    content_type = cgi.parse_header(http_env.get("CONTENT_TYPE"))
    charset = content_type[1].get('charset',None)
    if charset is None:
        charset = 'ascii'

    return input.read(length), charset


class ValidationError(Fault):
    pass


class WsgiMethodContext(MethodContext):
    def __init__(self, app, req_env, content_type):
        MethodContext.__init__(self, app)

        self.transport.type = 'wsgi'
        self.transport.req_env = req_env
        self.transport.resp_headers = {
            'Content-Type': content_type,
            'Content-Length': '0',
        }
        self.transport.req_method = req_env.get('REQUEST_METHOD', None)
        self.transport.wsdl_error = None


class WsgiApplication(ServerBase):
    transport = 'http://schemas.xmlsoap.org/soap/http'

    def __init__(self, app):
        ServerBase.__init__(self, app)

        self._allowed_http_verbs = app.in_protocol.allowed_http_verbs

    def __call__(self, req_env, start_response, wsgi_url=None):
        '''This method conforms to the WSGI spec for callable wsgi applications
        (PEP 333). It looks in environ['wsgi.input'] for a fully formed rpc
        message envelope, will deserialize the request parameters and call the
        method on the object returned by the get_handler() method.

        @param the http environment
        @param a callable that begins the response message
        @param the optional url

        @returns an iterable that contains the serialized response to the rpc
        message.
        '''

        url = wsgi_url
        if url is None:
            url = reconstruct_url(req_env).split('.wsdl')[0]

        if self.__is_wsdl_request(req_env):
            return self.__handle_wsdl_request(req_env, start_response, url)

        elif not (req_env['REQUEST_METHOD'].upper() in self._allowed_http_verbs):
            start_response(HTTP_405, [
                ('Content-type', ''),
                ('Allow', ', '.join(self._allowed_http_verbs)),
            ])
            return ['']

        else:
            return self.__handle_rpc(req_env, start_response)

    def __is_wsdl_request(self, req_env):
        # Get the wsdl for the service. Assume path_info matches pattern:
        # /stuff/stuff/stuff/serviceName.wsdl or
        # /stuff/stuff/stuff/serviceName/?wsdl

        return (
            req_env['REQUEST_METHOD'].lower() == 'get'
            and (
                   req_env['QUERY_STRING'].endswith('wsdl')
                or req_env['PATH_INFO'].endswith('wsdl')
            )
        )

    def __handle_wsdl_request(self, req_env, start_response, url):
        ctx = WsgiMethodContext(self.app, req_env, 'text/xml')

        try:
            wsdl = self.app.interface.get_interface_document()
            if wsdl is None:
                self.app.interface.build_interface_document(url)
                wsdl = self.app.interface.get_interface_document()

            assert wsdl != None

            self.event_manager.fire_event('wsdl',ctx) # implementation hook

            ctx.transport.resp_headers['Content-Length'] = str(len(wsdl))
            start_response(HTTP_200, ctx.transport.resp_headers.items())

            return [wsdl]

        except Exception, e:
            logger.exception(e)
            ctx.transport.wsdl_error = e
            # implementation hook
            self.event_manager.fire_event('wsdl_exception', ctx)

            start_response(HTTP_500, ctx.transport.resp_headers.items())

            return [""]

    def __handle_rpc(self, req_env, start_response):
        ctx = WsgiMethodContext(self.app, req_env,
                                                self.app.out_protocol.mime_type)

        # implementation hook
        self.event_manager.fire_event('wsgi_call', ctx)

        ctx.in_string, in_string_charset = reconstruct_wsgi_request(req_env)

        self.get_in_object(ctx, in_string_charset)

        return_code = HTTP_200
        if ctx.in_error:
            out_object = ctx.in_error
            return_code = HTTP_500

        else:
            if ctx.service_class == None:
                start_response(HTTP_404, ctx.transport.resp_headers.items())
                return ['']

            self.get_out_object(ctx)
            if not (ctx.out_error is None):
                return_code = HTTP_500

        self.get_out_string(ctx)
        if ctx.out_string is None:
            ctx.out_string = [""]

        # implementation hook
        self.event_manager.fire_event('wsgi_return', ctx)

        if ctx.descriptor and ctx.descriptor.mtom:
            # when there are more than one return type, the result is
            # encapsulated inside a list. when there's just one, the result
            # is returned unencapsulated. the apply_mtom always expects the
            # objects to be inside an iterable, hence the following test.
            out_type_info = ctx.descriptor.out_message._type_info
            if len(out_type_info) == 1:
                out_object = [out_object]

            ctx.transport.resp_headers, ctx.out_string = apply_mtom(
                    ctx.transport.resp_headers, ctx.out_string,
                    ctx.descriptor.out_message._type_info.values(),
                    out_object
                )

        # initiate the response
        del ctx.transport.resp_headers['Content-Length']
        start_response(return_code, ctx.transport.resp_headers.items())

        return ctx.out_string



#!/usr/bin/env python
#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('rpclib.protocol.soap')
logger.setLevel(logging.DEBUG)

from rpclib.application import Application
from rpclib.test.interop.server._service import services
from rpclib.protocol.csv import OutCsv
from rpclib.protocol.http import HttpRpc
from rpclib.interface.wsdl import Wsdl11

httprpc_csv_application = Application(services,
        'rpclib.test.interop.server.httprpc.csv', Wsdl11(), HttpRpc(), OutCsv())
from rpclib.server.wsgi import WsgiApplication

if __name__ == '__main__':
    try:
        from wsgiref.simple_server import make_server
        from wsgiref.validate import validator

        wsgi_application = WsgiApplication(httprpc_csv_application)
        server = make_server('0.0.0.0', 9755, validator(wsgi_application))

        logger.info('Starting interop server at %s:%s.' % ('0.0.0.0', 9755))
        logger.info('WSDL is at: /?wsdl')
        server.serve_forever()

    except ImportError:
        print "Error: example server code requires Python >= 2.5"




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from rpclib.model import ModelBase

import rpclib.const.xml_ns
_ns_soap_env = rpclib.const.xml_ns.soap_env
_pref_soap_env = rpclib.const.xml_ns.const_prefmap[_ns_soap_env]

class Fault(ModelBase, Exception):
    __type_name__ = "Fault"

    def __init__(self, faultcode='Server', faultstring="",
                 faultactor="", detail=None):
        if faultcode.startswith('%s:' % _pref_soap_env):
            self.faultcode = faultcode
        else:
            self.faultcode = '%s:%s' % (_pref_soap_env, faultcode)

        self.faultstring = faultstring or self.get_type_name()
        self.faultactor = faultactor
        self.detail = detail

    def __repr__(self):
        return "%s: %r" % (self.faultcode, self.faultstring)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from lxml import etree

import rpclib.const.xml_ns

_ns_xsd = rpclib.const.xml_ns.xsd

def fault_add(interface, cls):
    app = interface.app
    complex_type = etree.Element('{%s}complexType' % _ns_xsd)
    complex_type.set('name', '%sFault' % cls.get_type_name())

    extends = getattr(cls, '__extends__', None)
    if extends is not None:
        complex_content = etree.SubElement(complex_type,
                                        '{%s}complexContent' % _ns_xsd)
        extension = etree.SubElement(complex_content, "{%s}extension"
                                                                  % _ns_xsd)
        extension.set('base', extends.get_type_name_ns(app))
        sequence_parent = extension
    else:
        sequence_parent = complex_type

    etree.SubElement(sequence_parent, '{%s}sequence' % _ns_xsd)

    interface.add_complex_type(cls, complex_type)

    top_level_element = etree.Element('{%s}element' % _ns_xsd)
    top_level_element.set('name', cls.get_type_name())
    top_level_element.set('{%s}type' % _ns_xsd,
                          '%sFault' % cls.get_type_name_ns(app))

    interface.add_element(cls, top_level_element)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

import cgi
import traceback

import rpclib.const.xml_ns as ns

from lxml import etree

from rpclib.protocol.xml import XmlObject
from rpclib.protocol.soap.mime import collapse_swa

from rpclib.protocol import ProtocolBase
from rpclib.model.fault import Fault
from rpclib.model.primitive import string_encoding

class ValidationError(Fault):
    pass

def _from_soap(in_envelope_xml, xmlids=None):
    '''
    Parses the xml string into the header and payload
    '''

    if xmlids:
        resolve_hrefs(in_envelope_xml, xmlids)

    if in_envelope_xml.tag != '{%s}Envelope' % ns.soap_env:
        raise Fault('Client.SoapError', 'No {%s}Envelope element was found!' %
                                                            ns.soap_env)

    header_envelope = in_envelope_xml.xpath('e:Header',
                                          namespaces={'e': ns.soap_env})
    body_envelope = in_envelope_xml.xpath('e:Body',
                                          namespaces={'e': ns.soap_env})

    if len(header_envelope) == 0 and len(body_envelope) == 0:
        raise Fault('Client.SoapError', 'Soap envelope is empty!' %
                                                            ns.soap_env)

    header=None
    if len(header_envelope) > 0 and len(header_envelope[0]) > 0:
        header = header_envelope[0].getchildren()

    body=None
    if len(body_envelope) > 0 and len(body_envelope[0]) > 0:
        body = body_envelope[0].getchildren()[0]

    return header, body

def _parse_xml_string(xml_string, charset=None):
    try:
        if charset is None:
            charset = string_encoding

        root, xmlids = etree.XMLID(xml_string.decode(charset))

    except ValueError,e:
        logger.debug('%s -- falling back to str decoding.' % (e))
        root, xmlids = etree.XMLID(xml_string)

    return root, xmlids

# see http://www.w3.org/TR/2000/NOTE-SOAP-20000508/
# section 5.2.1 for an example of how the id and href attributes are used.
def resolve_hrefs(element, xmlids):
    for e in element:
        if e.get('id'):
            continue # don't need to resolve this element

        elif e.get('href'):
            resolved_element = xmlids[e.get('href').replace('#', '')]
            if resolved_element is None:
                continue
            resolve_hrefs(resolved_element, xmlids)

            # copies the attributes
            [e.set(k, v) for k, v in resolved_element.items()]

            # copies the children
            [e.append(child) for child in resolved_element.getchildren()]

            # copies the text
            e.text = resolved_element.text

        else:
            resolve_hrefs(e, xmlids)

    return element

def Soap11(validator=None):
    if validator is None:
        return _Soap11()
    else:
        return _Soap11Strict(validator=validator)

class _Soap11(XmlObject):
    class NO_WRAPPER:
        pass
    class IN_WRAPPER:
        pass
    class OUT_WRAPPER:
        pass

    allowed_http_verbs = ['POST']
    mime_type = 'application/soap+xml'

    def __init__(self, app=None):
        XmlObject.__init__(self, app)

        self.in_wrapper = _Soap11.IN_WRAPPER
        self.out_wrapper = _Soap11.OUT_WRAPPER

    def create_in_document(self, ctx, charset=None):
        if ctx.transport.type == 'wsgi':
            content_type = cgi.parse_header(ctx.transport.req_env.get("CONTENT_TYPE"))

            collapse_swa(content_type, ctx.in_string)

        ctx.in_document = _parse_xml_string(ctx.in_string, charset)

    def create_out_string(self, ctx, charset=None):
        """Sets an iterable of string fragments to ctx.out_string"""
        if charset is None:
            charset = string_encoding

        ctx.out_string = [etree.tostring(ctx.out_document, xml_declaration=True,
                                                              encoding=charset)]

    def decompose_incoming_envelope(self, ctx):
        envelope_xml, xmlids = ctx.in_document
        header_doc, body_doc = _from_soap(envelope_xml, xmlids)

        if len(body_doc) > 0 and body_doc.tag == '{%s}Fault' % ns.soap_env:
            ctx.in_body_doc = body_doc

        elif not (body_doc is None):
            try:
                self.validate(body_doc)
                if (not (body_doc is None) and
                                            (ctx.method_request_string is None)):
                    ctx.method_request_string = body_doc.tag
                    logger.debug("\033[92mMethod request_string: %r\033[0m" %
                                                    ctx.method_request_string)

            finally:
                # for performance reasons, we don't want the following to run
                # in production even though we won't see the results.
                # that's why one needs to explicitly set the logging level of
                # the 'rpclib.protocol.soap._base' to DEBUG to see the xml data.
                if logger.level == logging.DEBUG:
                    try:
                        logger.debug(etree.tostring(envelope_xml,
                                                             pretty_print=True))
                    except etree.XMLSyntaxError, e:
                        logger.debug(body_doc)
                        raise Fault('Client.Xml', 'Error at line: %d, '
                                    'col: %d' % e.position)

            if ctx.method_request_string is None:
                raise Exception("Could not extract method request string from "
                                "the request!")
            try:
                if ctx.service_class is None: # i.e. if it's a server
                    self.set_method_descriptor(ctx)

            except Exception,e:
                logger.debug(traceback.format_exc())
                raise ValidationError('Client', 'Method not found: %r' %
                                                    ctx.method_request_string)

            ctx.in_header_doc = header_doc
            ctx.in_body_doc = body_doc

    def deserialize(self, ctx):
        """Takes a MethodContext instance and a string containing ONE soap
        message.
        Returns the corresponding native python object

        Not meant to be overridden.
        """

        if ctx.in_body_doc.tag == "{%s}Fault" % ns.soap_env:
            ctx.in_object = None
            ctx.in_error = self.from_element(Fault, ctx.in_body_doc)

        else:
            if self.in_wrapper is self.IN_WRAPPER:
                header_class = ctx.descriptor.in_header
                body_class = ctx.descriptor.in_message

            elif self.in_wrapper is self.OUT_WRAPPER:
                header_class = ctx.descriptor.out_header
                body_class = ctx.descriptor.out_message

            # decode header objects
            if (ctx.in_header_doc is not None and header_class is not None):
                if isinstance(header_class, (list, tuple)):
                    headers = [None] * len(header_class)
                    for i, (header_doc, head_class) in enumerate(
                                          zip(ctx.in_header_doc, header_class)):
                        if len(header_doc) > 0:
                            headers[i] = self.from_element(head_class, header_doc)
                    ctx.in_header = tuple(headers)

                else:
                    header_doc = ctx.in_header_doc[0]
                    if len(header_doc) > 0:
                        ctx.in_header = self.from_element(header_class, header_doc)

            # decode method arguments
            if ctx.in_body_doc is not None and len(ctx.in_body_doc) > 0:
                ctx.in_object = self.from_element(body_class, ctx.in_body_doc)
            else:
                ctx.in_object = [None] * len(body_class._type_info)

        self.event_manager.fire_event('deserialize', ctx)

    def serialize(self, ctx):
        """Uses ctx.out_object, ctx.out_header or ctx.out_error to set
        ctx.out_body_doc, ctx.out_header_doc and ctx.out_document as an
        lxml.etree._Element instance.

        Not meant to be overridden.
        """

        # construct the soap response, and serialize it
        nsmap = self.app.interface.nsmap
        ctx.out_document = etree.Element('{%s}Envelope' % ns.soap_env,
                                                                    nsmap=nsmap)

        if not (ctx.out_error is None):
            # FIXME: There's no way to alter soap response headers for the user.
            ctx.out_body_doc = out_body_doc = etree.SubElement(ctx.out_document,
                            '{%s}Body' % ns.soap_env, nsmap=nsmap)
            self.to_parent_element(ctx.out_error.__class__, ctx.out_error,
                                    self.app.interface.get_tns(), out_body_doc)

            if logger.level == logging.DEBUG:
                logger.debug(etree.tostring(ctx.out_document, pretty_print=True))

        else:
            # header
            if ctx.out_header is not None:
                if self.out_wrapper is self.OUT_WRAPPER:
                    header_message_class = ctx.descriptor.out_header
                else:
                    header_message_class = ctx.descriptor.in_header

                if ctx.descriptor.out_header is None:
                    logger.warning(
                        "Skipping soap response header as %r method is not "
                        "declared to have one." % ctx.method_name)

                else:
                    ctx.out_header_doc = soap_header_elt = etree.SubElement(
                                    ctx.out_document, '{%s}Header' % ns.soap_env)

                    if isinstance(header_message_class, (list, tuple)):
                        if isinstance(ctx.out_header, (list, tuple)):
                            out_headers = ctx.out_header
                        else:
                            out_headers = (ctx.out_header,)

                        for header_class, out_header in zip(header_message_class,
                                                                out_headers):
                            self.to_parent_element(header_class,
                                out_header,
                                self.app.interface.get_tns(),
                                soap_header_elt,
                                header_class.get_type_name(),
                            )
                    else:
                        self.to_parent_element(header_message_class,
                            ctx.out_header,
                            self.app.interface.get_tns(),
                            soap_header_elt,
                            header_message_class.get_type_name()
                        )

            # body
            ctx.out_body_doc = out_body_doc = etree.SubElement(ctx.out_document,
                                               '{%s}Body' % ns.soap_env)

            # instantiate the result message
            if self.out_wrapper is self.NO_WRAPPER:
                result_message_class = ctx.descriptor.in_message
                result_message = ctx.out_object

            else:
                if self.out_wrapper is self.IN_WRAPPER:
                    result_message_class = ctx.descriptor.in_message
                elif self.out_wrapper is self.OUT_WRAPPER:
                    result_message_class = ctx.descriptor.out_message

                result_message = result_message_class()

                # assign raw result to its wrapper, result_message
                out_type_info = result_message_class._type_info

                if len(out_type_info) == 1:
                    attr_name = result_message_class._type_info.keys()[0]
                    setattr(result_message, attr_name, ctx.out_object)

                else:
                    for i in range(len(out_type_info)):
                        attr_name=result_message_class._type_info.keys()[i]
                        setattr(result_message, attr_name, ctx.out_object[i])

            # transform the results into an element
            self.to_parent_element(result_message_class,
                  result_message, self.app.interface.get_tns(), out_body_doc)

            if logger.level == logging.DEBUG:
                logger.debug('\033[91m'+ "Response" + '\033[0m')
                logger.debug(etree.tostring(ctx.out_document,
                                       xml_declaration=True, pretty_print=True))

        self.event_manager.fire_event('serialize',ctx)

class _Soap11Strict(_Soap11):
    def __init__(self, app=None, validator='lxml'):
        """Soap 1.1 Protocol with validators.

        @param A rpclib.application.Application instance.
        @param The validator to use. Currently the only supported value is 'lxml'
        """

        if validator == 'lxml':
            self.validate = self.__validate_lxml
        else:
            raise ValueError(validator)

        _Soap11.__init__(self, app)

    def set_app(self, value):
        _Soap11.set_app(self, value)

        self.validation_schema = None

        if value:
            from rpclib.interface.wsdl import Wsdl11

            wsdl = Wsdl11(value)
            wsdl.build_validation_schema()

            self.validation_schema = wsdl.validation_schema

    def __validate_lxml(self, payload):
        ret = self.validation_schema.validate(payload)

        logger.debug("Validated ? %s" % str(ret))
        if ret == False:
            raise ValidationError('Client.SchemaValidation',
                               str(self.validation_schema.error_log.last_error))




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""cdict (ClassDict) is a funny kind of dict that tries to return the values for
the base classes of a key when the entry for the key is not found.
"""

import logging
logger = logging.getLogger(__name__)

class cdict(dict):
    def __getitem__(self, cls):
        try:
            return dict.__getitem__(self, cls)

        except KeyError, e:
            try:
                return dict.__getitem__(self, cls._is_clone_of)
            except AttributeError:
                pass
            except KeyError:
                pass

            for b in cls.__bases__:
                return self[b]
            raise




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

from lxml import etree

from rpclib.model.complex import XMLAttribute # FIXME: Rename this to XmlAttribute

from _base import nillable_value
from _base import nillable_element

def get_members_etree(prot, cls, inst, parent):
    parent_cls = getattr(cls, '__extends__', None)
    if not (parent_cls is None):
        get_members_etree(prot, parent_cls, inst, parent)

    for k, v in cls._type_info.items():
        try:
            subvalue = getattr(inst, k, None)
        except: # to guard against sqlalchemy throwing NoSuchColumnError
            subvalue = None

        if isinstance(v, XMLAttribute):
            v.marshall(k, subvalue, parent)
            continue

        mo = v.Attributes.max_occurs
        if mo == 'unbounded' or mo > 1:
            if subvalue != None:
                for sv in subvalue:
                    prot.to_parent_element(v, sv, cls.get_namespace(), parent, k)

        # Don't include empty values for non-nillable optional attributes.
        elif subvalue is not None or v.Attributes.nillable or v.Attributes.min_occurs > 0:
            prot.to_parent_element(v, subvalue, cls.get_namespace(), parent, k)


@nillable_value
def complex_to_parent_element(prot, cls, value, tns, parent_elt, name=None):
    if name is None:
        name = cls.get_type_name()

    element = etree.SubElement(parent_elt, "{%s}%s" % (tns, name))

    # here, we try our best to match the incoming value with the class
    # definition in cls._type_map.
    inst = cls.get_serialization_instance(value)

    get_members_etree(prot, cls, inst, element)

@nillable_element
def complex_from_element(prot, cls, element):
    #import pdb; pdb.set_trace()
    inst = cls.get_deserialization_instance()

    # FIXME: the result of this method should be cached when build_wsdl is
    #        called (i.e. when _type_info becomes by definition immutable).
    flat_type_info = cls.get_flat_type_info(cls)

    # initialize instance
    for k in flat_type_info:
        setattr(inst, k, None)

    # parse input to set incoming data to related attributes.
    for c in element:
        if isinstance(c, etree._Comment):
            continue

        key = c.tag.split('}')[-1]

        member = flat_type_info.get(key, None)
        if member is None:
            continue

        if isinstance(member, XMLAttribute):
            value = element.get(key)

        else:
            mo = member.Attributes.max_occurs
            if mo == 'unbounded' or mo > 1:
                value = getattr(inst, key, None)
                if value is None:
                    value = []

                value.append(prot.from_element(member, c))

            else:
                value = prot.from_element(member, c)

        setattr(inst, key, value)

    return inst

@nillable_element
def array_from_element(prot, cls, element):
    retval = [ ]
    (serializer,) = cls._type_info.values()

    for child in element.getchildren():
        retval.append(prot.from_element(serializer, child))

    return retval

@nillable_element
def iterable_from_element(cls, element):
    (serializer,) = cls._type_info.values()

    for child in element.getchildren():
        yield serializer.from_xml(child)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import rpclib.const.xml_ns
_ns_xs = rpclib.const.xml_ns.xsd

from lxml import etree

from rpclib.model.primitive import String
from rpclib.model.primitive import Decimal

from _base import simple_get_restriction_tag

def string_get_restriction_tag(interface, cls):
    restriction = simple_get_restriction_tag(interface, cls)

    # length
    if cls.Attributes.min_len == cls.Attributes.max_len:
        length = etree.SubElement(restriction, '{%s}length' % _ns_xs)
        length.set('value', str(cls.Attributes.min_len))

    else:
        if cls.Attributes.min_len != String.Attributes.min_len:
            min_l = etree.SubElement(restriction, '{%s}minLength' % _ns_xs)
            min_l.set('value', str(cls.Attributes.min_len))

        if cls.Attributes.max_len != String.Attributes.max_len:
            max_l = etree.SubElement(restriction, '{%s}maxLength' % _ns_xs)
            max_l.set('value', str(cls.Attributes.max_len))

    # pattern
    if cls.Attributes.pattern != String.Attributes.pattern:
        pattern = etree.SubElement(restriction, '{%s}pattern' % _ns_xs)
        pattern.set('value', cls.Attributes.pattern)

    return restriction

def decimal_get_restriction_tag(interface, cls):
    restriction = simple_get_restriction_tag(interface, cls)

    if cls.Attributes.gt != Decimal.Attributes.gt:
        min_l = etree.SubElement(restriction, '{%s}minExclusive' % _ns_xs)
        min_l.set('value', str(cls.Attributes.gt))

    if cls.Attributes.ge != Decimal.Attributes.ge:
        min_l = etree.SubElement(restriction, '{%s}minInclusive' % _ns_xs)
        min_l.set('value', str(cls.Attributes.ge))

    if cls.Attributes.lt != Decimal.Attributes.lt:
        min_l = etree.SubElement(restriction, '{%s}maxExclusive' % _ns_xs)
        min_l.set('value', str(cls.Attributes.lt))

    if cls.Attributes.le != Decimal.Attributes.le:
        min_l = etree.SubElement(restriction, '{%s}maxInclusive' % _ns_xs)
        min_l.set('value', str(cls.Attributes.le))

    return restriction




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import os.path

import rpclib.interface.wsdl
import rpclib.protocol.soap

from lxml import etree

from rpclib.application import Application
from rpclib.service import ServiceBase
from rpclib.decorator import rpc
from rpclib.const import xml_ns as ns

class XSDGenerator(object):
    '''Class to support xsd generation for rpclib models.'''

    # Simplified schema mapping used for building standalone XSDs without SOAP
    # specific namespace imports
    model_schema_nsmap = {
        'xs': ns.xsd,
        'xsi': ns.xsi,
        'xop': ns.xop,
    }

    __ct_string = '{%s}complexType' % ns.xsd
    __el_string = '{%s}element' % ns.xsd
    __imp_string = '{%s}import' % ns.xsd

    def __get_binding_service(self, model):
        '''A factory method to create a simple service class.

        Builds a class based on ServiceBase to bind an arbritary rpclib
        class serilizer model to an instance of a rpclib application.
        @param A rpclib ComplexModel model
        '''

        class BindingService(ServiceBase):
            @rpc(model)
            def binding_method(self, model):
                pass

        return BindingService

    def __get_binding_application(self, binding_service):
        '''Builds an instance of rpclib.Application

        The Application built is populated with an instance of a Service Class
        based on ServiceBase
        @param A class based on ServiceBase
        '''

        binding_application = Application([binding_service],
                                          rpclib.interface.wsdl.Wsdl11,
                                          rpclib.protocol.soap.Soap11,
                                          tns='binding_application')

        # The lxml Element nsmap is being overridden to remove the unneeded
        # namespaces
        binding_application.nsmap = XSDGenerator.model_schema_nsmap
        binding_application.prefmap = \
                dict([(b,a) for a,b in XSDGenerator.model_schema_nsmap.items()])

        binding_application.service_mapping = {}

        return binding_application

    def __get_nodes(self, model):
        '''Builds and returns the scheame nodes as a python dictionary

        @param A rpclib ComplexModel model
        '''

        binding_service = self.__get_binding_service(model)
        app = self.__get_binding_application(binding_service)
        nodes = app.interface.build_schema_nodes(types=None)
        return nodes

    def __get_model_node(self, model, nodes):
        '''Iterate over a dict of Elements to locate the correct type'''

        xsd_out = None

        for node in nodes.values():

            for element in node.iterfind(XSDGenerator.__ct_string):
                if element.attrib['name'] == model.get_type_name():
                    xsd_out = node
                    break

        # The correct node should have been set, if not an exception needs
        # to be raised
        if xsd_out is None:
            raise KeyError('Element not set; model name attribue not found')

        return xsd_out

    def __get_xsd_file_name(self, model, model_node):
        '''Returns the correct xsd name for a single model.'''

        file_prefix = None

        for el in model_node.iterfind(XSDGenerator.__el_string):
            if el.attrib['type'].find(model.get_type_name()) !=-1:
                marker = el.attrib['type'].find(':')
                file_prefix = el.attrib['type'][:marker]
                break


        if file_prefix is None:
            raise ValueError('Unable to set the file prefix')

        return '{0:>s}.xsd'.format(file_prefix)

    def __write_xsd(self, encoding, file_path, xsd_out_node):
        '''Writes the supplied schema node to file

        @param The string encoding
        @param The file path for the file
        @param A Element representing the xsd node
        '''

        f = open(file_path, 'w')

        etree.ElementTree(xsd_out_node).write(f,
            pretty_print=True,
            encoding=encoding,
            xml_declaration=True
        )

        f.close()


    def __clean_soap_nodes(self, element_dict):
        '''Strips soap specific elements and returns a list of elements.'''

        out_elements = []
        for element in element_dict.values():
            if element.attrib['targetNamespace'] != 'binding_application':
                out_elements.append(element)

        return out_elements


    def get_model_xsd(self, model, encoding='utf-8', pretty_print=False):
        '''Returns a string representation of an XSD for the specified model.

        @param  A rpclib.model class that will be represented in the schema.
        @param  The model's encoding.
        @param Boolean value to control if pretty printing should be used when
        returning the xsd as string.
        '''

        nodes = self.__get_nodes(model)
        xsd_out = self.__get_model_node(model, nodes)

        return etree.tostring(
                xsd_out,
                pretty_print=pretty_print,
                encoding=encoding,
                xml_declaration=True
                )


    def get_all_models_xsd(self, model, encoding='utf-8', pretty_print=False):
        '''Returns all related models as a list of strings

        @param A ClasserSerializer model
        @param  The models encoding.
        @param Boolean value to control if pretty printing should be used when
        returning the xsd as string.
        '''

        nodes = self.__get_nodes(model)

        elements = self.__clean_soap_nodes(nodes)
        string_elements = []
        for el in elements:
            string_elements.append(
                    etree.tostring(el, encoding=encoding, pretty_print=pretty_print)
                    )

        return string_elements



    def write_model_xsd_file(self, model, path, encoding='utf-8'):
        '''Builds a stand alone xsd file per model; returs the file name.

        @param The model
        @param The path (folder/directory location) for the file to be written
        @param The string encoding.
        '''

        if not os.path.isdir(path):
            raise IOError('Path does not exist')

        xsd_out_node = self.__get_model_node(model, self.__get_nodes(model))

        file_name =  self.__get_xsd_file_name(model, xsd_out_node)
        file_path = os.path.join(path, file_name)

        self.__write_xsd(encoding, file_path, xsd_out_node)

        return file_path


    def write_all_models(self, model, path, encoding='utf-8'):
        '''Writes a family of schemas to disk; returns a list of file names

        Writes the schema for the supplied model to disk along with files for
        any additional models that the supplied model depends on.

        @param A ClasserSerializer model
        @param The path (folder/directory location) where thefiles will be
        written
        @param The string encoding.
        '''

        if not os.path.isdir(path):
            raise IOError('Path does not exist')

        nodes = self.__get_nodes(model)

        out_file_list = []

        for k,v in nodes.items():
            if v.attrib['targetNamespace'] != 'binding_application':
                file_name = "{0:>s}.xsd".format(k)
                file_path = os.path.join(path, file_name)

                self.__write_xsd(encoding, file_path, v)
                out_file_list.append(file_path)

        return out_file_list




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""A soap server that uses http as transport, and wsgi as bridge api"""

import logging
logger = logging.getLogger(__name__)

from rpclib.model.fault import Fault
from rpclib._base import EventManager

HTTP_500 = '500 Internal server error'
HTTP_200 = '200 OK'
HTTP_405 = '405 Method Not Allowed'

class InternalError(Fault):
    pass

class ValidationError(Fault):
    pass

class ServerBase(object):
    transport = None

    def __init__(self, app):
        self.app = app
        self.app.transport = self.transport
        self.event_manager = EventManager(self)

    def get_in_object(self, ctx, in_string_charset=None):
        self.app.in_protocol.create_in_document(ctx, in_string_charset)

        try:
            # sets the ctx.in_body_doc and ctx.in_header_doc properties
            self.app.in_protocol.decompose_incoming_envelope(ctx)

            if ctx.service_class != None:
                ctx.service_class.event_manager.fire_event('decompose_envelope',
                                                                        ctx)
            self.app.in_protocol.deserialize(ctx)

        except Fault,e:
            ctx.in_object = None
            ctx.in_error = e
            ctx.out_error = e

    def get_out_object(self, ctx):
        self.app.process_request(ctx)

    def get_out_string(self, ctx):
        assert ctx.out_document is None
        assert ctx.out_string is None

        self.app.out_protocol.serialize(ctx)

        if ctx.service_class != None:
            if ctx.out_error is None:
                ctx.service_class.event_manager.fire_event(
                                            'method_return_document', ctx)
            else:
                ctx.service_class.event_manager.fire_event(
                                            'method_exception_document', ctx)

        self.app.out_protocol.create_out_string(ctx)

        if ctx.service_class != None:
            if ctx.out_error is None:
                ctx.service_class.event_manager.fire_event(
                                            'method_return_string', ctx)
            else:
                ctx.service_class.event_manager.fire_event(
                                            'method_exception_string', ctx)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

import shutil
import tempfile

import rpclib.const.xml_ns

from lxml import etree

from rpclib.util.cdict import cdict

from rpclib.interface import InterfaceBase
from rpclib.model import SimpleModel
from rpclib.model.primitive import String
from rpclib.model.primitive import Decimal
from rpclib.model.complex import ComplexModelBase
from rpclib.model.enum import EnumBase
from rpclib.model.fault import Fault

from rpclib.interface.xml_schema.model import simple_add
from rpclib.interface.xml_schema.model.complex import complex_add
from rpclib.interface.xml_schema.model.fault import fault_add
from rpclib.interface.xml_schema.model.enum import enum_add

from rpclib.interface.xml_schema.model import simple_get_restriction_tag
from rpclib.interface.xml_schema.model.primitive import string_get_restriction_tag
from rpclib.interface.xml_schema.model.primitive import decimal_get_restriction_tag

_add_handlers = cdict({
    object: lambda self,cls: None,
    SimpleModel: simple_add,
    ComplexModelBase: complex_add,
    Fault: fault_add,
    EnumBase: enum_add,
})

_get_restriction_tag_handlers = cdict({
    object: lambda self,cls: None,
    SimpleModel: simple_get_restriction_tag,
    String: string_get_restriction_tag,
    Decimal: decimal_get_restriction_tag,
})

_ns_xsd = rpclib.const.xml_ns.xsd
_ns_wsa = rpclib.const.xml_ns.wsa
_ns_wsdl = rpclib.const.xml_ns.wsdl
_ns_soap = rpclib.const.xml_ns.soap
_pref_wsa = rpclib.const.xml_ns.const_prefmap[_ns_wsa]

class XmlSchema(InterfaceBase):
    def __init__(self, app, import_base_namespaces=False):
        self.schema_dict = {}
        self.validation_schema = None
        self.import_base_namespaces = import_base_namespaces

        InterfaceBase.__init__(self, app)

    def add(self, cls):
        handler = _add_handlers[cls]
        return handler(self, cls)

    def get_restriction_tag(self, cls):
        handler = _get_restriction_tag_handlers[cls]
        return handler(self, cls)

    def build_schema_nodes(self, with_schema_location=False):
        self.schema_dict = {}

        for pref in self.namespaces:
            schema = self.get_schema_node(pref)

            # append import tags
            for namespace in self.imports[pref]:
                import_ = etree.SubElement(schema, "{%s}import" % _ns_xsd)
                import_.set("namespace", namespace)
                if with_schema_location:
                    import_.set('schemaLocation', "%s.xsd" %
                                           self.get_namespace_prefix(namespace))

                sl = rpclib.const.xml_ns.schema_location.get(namespace, None)
                if not (sl is None):
                    import_.set('schemaLocation', sl)

            # append simpleType and complexType tags
            for node in self.namespaces[pref].types.values():
                schema.append(node)

            # append element tags
            for node in self.namespaces[pref].elements.values():
                schema.append(node)

    def build_validation_schema(self):
        """Build application schema specifically for xml validation purposes.
        """

        self.build_schema_nodes(with_schema_location=True)

        pref_tns = self.get_namespace_prefix(self.get_tns())
        logger.debug("generating schema for targetNamespace=%r, prefix: %r"
                                                   % (self.get_tns(), pref_tns))

        tmp_dir_name = tempfile.mkdtemp()

        # serialize nodes to files
        for k,v in self.schema_dict.items():
            file_name = '%s/%s.xsd' % (tmp_dir_name, k)
            f = open(file_name, 'w')
            etree.ElementTree(v).write(f, pretty_print=True)
            f.close()
            logger.debug("writing %r for ns %s" % (file_name, self.nsmap[k]))

        f = open('%s/%s.xsd' % (tmp_dir_name, pref_tns), 'r')

        logger.debug("building schema...")
        self.validation_schema = etree.XMLSchema(etree.parse(f))

        logger.debug("schema %r built, cleaning up..." % self.validation_schema)
        f.close()

        shutil.rmtree(tmp_dir_name)
        logger.debug("removed %r" % tmp_dir_name)

    def get_schema_node(self, pref):
        """Return schema node for the given namespace prefix.

        types == None means the call is for creating a standalone xml schema
                      file for one single namespace.
        types != None means the call is for creating the wsdl file.
        """

        # create schema node
        if not (pref in self.schema_dict):
            schema = etree.Element("{%s}schema" % _ns_xsd, nsmap=self.nsmap)

            schema.set("targetNamespace", self.nsmap[pref])
            schema.set("elementFormDefault", "qualified")

            self.schema_dict[pref] = schema

        else:
            schema = self.schema_nodes[pref]

        return schema

    def get_interface_document(self):
        return self.schema_dict

    def build_interface_document(self, url):
        self.build_schema_nodes()

    def add_element(self, cls, node):
        pref = cls.get_namespace_prefix(self)

        schema_info = self.get_schema_info(pref)
        schema_info.elements[cls.get_type_name()] = node

    def add_simple_type(self, cls, node):
        ns = cls.get_namespace()
        tn = cls.get_type_name()
        pref = cls.get_namespace_prefix(self)

        self.__check_imports(cls, node)
        schema_info = self.get_schema_info(pref)
        schema_info.types[tn] = node

        class_key = '{%s}%s' % (ns,tn)
        logger.debug('\tadding class %r for %r' % (repr(cls), class_key))

        self.classes[class_key] = cls
        if ns == self.get_tns():
            self.classes[tn] = cls

    def add_complex_type(self, cls, node):
        ns = cls.get_namespace()
        tn = cls.get_type_name()
        pref = cls.get_namespace_prefix(self)

        self.__check_imports(cls, node)
        schema_info = self.get_schema_info(pref)
        schema_info.types[tn] = node

        class_key = '{%s}%s' % (ns,tn)
        logger.debug('\tadding class %r for %r' % (repr(cls), class_key))

        self.classes[class_key] = cls
        if ns == self.get_tns():
            self.classes[tn] = cls

    def reset_interface(self):
        InterfaceBase.reset_interface(self)

        self.nsmap = dict(rpclib.const.xml_ns.const_nsmap)
        self.prefmap = dict(rpclib.const.xml_ns.const_prefmap)

        self.nsmap['tns'] = self.get_tns()
        self.prefmap[self.get_tns()] = 'tns'

    def has_class(self, cls):
        ns_prefix = cls.get_namespace_prefix(self)
        if ns_prefix in rpclib.const.xml_ns.const_nsmap:
            return True

        else:
            return InterfaceBase.has_class(self, cls)

    # FIXME: this is an ugly hack. we need proper dependency management
    def __check_imports(self, cls, node):
        pref_tns = cls.get_namespace_prefix(self)

        def is_valid_import(pref):
            return pref != pref_tns and (
                    self.import_base_namespaces or
                    (not (pref in rpclib.const.xml_ns.const_nsmap))
                )

        if not (pref_tns in self.imports):
            self.imports[pref_tns] = set()

        for c in node:
            if c.tag == "{%s}complexContent" % _ns_xsd:
                extension = c.getchildren()[0]

                if extension.tag == '{%s}extension' % _ns_xsd:
                    pref = extension.attrib['base'].split(':')[0]
                    if is_valid_import(pref):
                        self.imports[pref_tns].add(self.nsmap[pref])
                    seq = extension.getchildren()[0]

                else:
                    seq = c.getchildren()[0]

            else:
                seq = c

            if seq.tag == '{%s}sequence' % _ns_xsd:
                for e in seq:
                    pref = e.attrib['type'].split(':')[0]
                    if is_valid_import(pref):
                        self.imports[pref_tns].add(self.nsmap[pref])

            elif seq.tag == '{%s}restriction' % _ns_xsd:
                pref = seq.attrib['base'].split(':')[0]
                if is_valid_import(pref):
                    self.imports[pref_tns].add(self.nsmap[pref])

            elif seq.tag == '{%s}attribute' % _ns_xsd:
                typ = seq.get('type', '')
                t_pref = typ.split(':')[0]

                if t_pref and is_valid_import(t_pref):
                    self.imports[pref_tns].add(self.nsmap[t_pref])

                ref = seq.get('ref', '')
                r_pref = ref.split(':')[0]

                if r_pref and is_valid_import(r_pref):
                    self.imports[pref_tns].add(self.nsmap[r_pref])

            else:
                raise Exception("i guess you need to hack some more")




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""A server that doesn't support any transport at all -- it's implemented for
testing services without running a server.
"""

from rpclib.client._base import Factory
from rpclib._base import MethodContext
from rpclib.server import ServerBase

class NullServer(ServerBase):
    transport = 'noconn://null.rpclib'

    def __init__(self, app):
        ServerBase.__init__(self, app)

        self.service = _FunctionProxy(self, self.app)
        self.factory = Factory(self.app)

    def get_wsdl(self):
        return self.app.get_interface_document(self.url)

    def set_options(self, **kwargs):
        self.service.in_header = kwargs.get('soapheaders', None)

class _FunctionProxy(object):
    def __init__(self, parent, app):
        self.__app = app
        self.in_header = None

    def __getattr__(self, key):
        return _FunctionCall(self.__app, key, self.in_header)

class _FunctionCall(object):
    def __init__(self, app, key, in_header):
        self.__key = key
        self.__app = app
        self.__in_header = in_header

    def __call__(self, *args, **kwargs):
        ctx = MethodContext(self.__app)
        ctx.method_request_string = self.__key
        ctx.in_header = self.__in_header
        ctx.in_object = args

        self.__app.in_protocol.set_method_descriptor(ctx)
        self.__app.process_request(ctx)

        if ctx.out_error:
            raise ctx.out_error
        else:
            # workaround to have the context be disposed when the caller is done
            # with the return value. the context is sometimes needed to fully
            # construct the return object.
            try:
                ctx.out_object.__ctx__ = ctx
            except AttributeError,e:
                # not all objects let this happen. (eg. built-in types like str)
                # which don't need the context anyway.
                pass

            return ctx.out_object




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

import base64

from lxml import etree

from rpclib.model.binary import Attachment

from _base import nillable_value
from _base import nillable_element

@nillable_value
def to_parent_element(cls, value, tns, parent_elt, name='retval'):
    '''This class method takes the data from the attachment and
    base64 encodes it as the text of an Element. An attachment can
    specify a file_name and if no data is given, it will read the data
    from the file
    '''

    element = etree.SubElement(parent_elt, '{%s}%s' % (tns,name))
    element.text = base64.encodestring(cls.to_string(value))

@nillable_element
def from_xml(cls, element):
    '''This method returns an Attachment object that contains
    the base64 decoded string of the text of the given element
    '''
    data = base64.decodestring(element.text)
    a = Attachment(data=data)
    return a





#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import rpclib.const.xml_ns

def nillable_dict(func):
    def wrapper(cls, element):
        if element is None:
            return {}
        else:
            return func(cls, element)
    return wrapper

def nillable_string(func):
    def wrapper(cls, string):
        if string is None:
            return None
        else:
            return func(cls, string)
    return wrapper

class ModelBase(object):
    __namespace__ = None
    __type_name__ = None

    # These are not the xml schema defaults. The xml schema defaults are
    # considered in ComplexModel's add_to_schema method. the defaults here
    # are to reflect what people seem to want most.
    class Attributes(object):
        nillable = True
        min_occurs = 0
        max_occurs = 1

    class Annotations(object):
        doc = ""

    class Empty(object):
        pass

    @staticmethod
    def is_default(cls):
        return True

    @classmethod
    def get_namespace_prefix(cls, interface):
        ns = cls.get_namespace()

        retval = interface.get_namespace_prefix(ns)

        return retval

    @classmethod
    def get_namespace(cls):
        return cls.__namespace__

    @staticmethod
    def resolve_namespace(cls, default_ns):
        if cls.__namespace__ is rpclib.const.xml_ns.DEFAULT_NS:
            cls.__namespace__ = default_ns

        if (cls.__namespace__ in rpclib.const.xml_ns.const_prefmap and
                                                       not cls.is_default(cls)):
            cls.__namespace__ = default_ns

        if cls.__namespace__ is None:
            cls.__namespace__ = cls.__module__

    @classmethod
    def get_type_name(cls):
        retval = cls.__type_name__
        if retval is None:
            retval = cls.__name__

        return retval

    @classmethod
    def get_type_name_ns(cls, app):
        if cls.get_namespace() != None:
            return "%s:%s" % (cls.get_namespace_prefix(app), cls.get_type_name())

    @classmethod
    @nillable_string
    def to_string(cls, value):
        return str(value)

    @classmethod
    @nillable_string
    def to_dict(cls, value):
        return {cls.get_type_name(): cls.to_string(value)}

    @classmethod
    def customize(cls, ** kwargs):
        cls_name, cls_bases, cls_dict = cls._s_customize(cls, ** kwargs)

        return type(cls_name, cls_bases, cls_dict)

    @staticmethod
    def _s_customize(cls, ** kwargs):
        """This function duplicates and customizes the class it belongs to. The
        original class remains unchanged.
        """

        cls_dict = {}

        #assert (issubclass(cls, rpclib.model._base.SimpleModel)
        #            or cls.__name__ == 'Array'
        #            or getattr(cls, '_type_info', None) is not None
        #        ),(cls.__name__)

        for k in cls.__dict__:
            if not (k in ("__dict__", "__weakref__")):
                cls_dict[k] = cls.__dict__[k]

        class Attributes(cls.Attributes):
            pass
        cls_dict['Attributes'] = Attributes

        class Annotations(cls.Annotations):
            pass
        cls_dict['Annotations'] = Annotations

        if not ('_is_clone_of' in cls_dict):
            cls_dict['_is_clone_of'] = cls

        for k, v in kwargs.items():
            if k != "doc":
                setattr(Attributes, k, v)
            else:
                setattr(Annotations, k, v)

        return (cls.__name__, cls.__bases__, cls_dict)

class Null(ModelBase):
    @classmethod
    def to_string(cls, value):
        return ""

    @classmethod
    def from_string(cls, value):
        return None

class SimpleModel(ModelBase):
    __namespace__ = "http://www.w3.org/2001/XMLSchema"
    __base_type__ = None

    class Attributes(ModelBase.Attributes):
        values = set()

    def __new__(cls, ** kwargs):
        """
        Overriden so that any attempt to instantiate a primitive will return a
        customized class instead of an instance.

        See rpclib.model.base.ModelBase for more information.
        """

        retval = cls.customize( ** kwargs)

        if not retval.is_default(retval):
            retval.__base_type__ = cls
            retval.__type_name__ = kwargs.get("type_name", ModelBase.Empty)

        return retval

    @staticmethod
    def is_default(cls):
        return (cls.Attributes.values == SimpleModel.Attributes.values)



#!/usr/bin/env python
#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import unittest

from rpclib.util.cdict import cdict

class TestClassDict(unittest.TestCase):
    def test_cdict(self):
        d = cdict()

        class A(object):
            pass

        class B(A):
            pass

        class C(A):
            pass

        class D(object):
            pass

        d[A] = 1
        d[C] = 2

        self.assertEquals(d[A], 1)
        self.assertEquals(d[B], 1)
        self.assertEquals(d[C], 2)

        try:
            d[D]
            raise Exception("Must fail")
        except KeyError:
            pass

        try:
            d[object]
            raise Exception("Must fail")
        except KeyError:
            pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import datetime
import unittest

import rpclib.interface.wsdl
import rpclib.protocol.soap

from lxml import etree

from rpclib.application import Application
from rpclib.decorator import rpc
from rpclib.service import ServiceBase
from rpclib.model.complex import Array
from rpclib.model.complex import ComplexModel
from rpclib.model.primitive import DateTime
from rpclib.model.primitive import Float
from rpclib.model.primitive import Integer
from rpclib.model.primitive import String

Application.transport = 'test'


class Address(ComplexModel):
    __namespace__ = "TestService"

    street = String
    city = String
    zip = Integer
    since = DateTime
    laditude = Float
    longitude = Float

class Person(ComplexModel):
    __namespace__ = "TestService"

    name = String
    birthdate = DateTime
    age = Integer
    addresses = Array(Address)
    titles = Array(String)

class Request(ComplexModel):
    __namespace__ = "TestService"

    param1 = String
    param2 = Integer

class Response(ComplexModel):
    __namespace__ = "TestService"

    param1 = Float

class TypeNS1(ComplexModel):
    __namespace__ = "TestService.NS1"

    s = String
    i = Integer

class TypeNS2(ComplexModel):
    __namespace__ = "TestService.NS2"

    d = DateTime
    f = Float

class MultipleNamespaceService(ServiceBase):
    @rpc(TypeNS1, TypeNS2)
    def a(self, t1, t2):
        return "OK"

class MultipleNamespaceValidatingService(MultipleNamespaceService):
    def __init__(self):
        MultipleNamespaceService.__init__(self)

        self.validating_service = True

class TestService(ServiceBase):
    @rpc(String, _returns=String)
    def aa(self, s):
        return s

    @rpc(String, Integer, _returns=DateTime)
    def a(self, s, i):
        return datetime.datetime.now()

    @rpc(Person, String, Address, _returns=Address)
    def b(self, p, s, a):
        return Address()

    @rpc(Person, isAsync=True)
    def d(self, Person):
        pass

    @rpc(Person, isCallback=True)
    def e(self, Person):
        pass

    @rpc(String, String, String, _returns=String,
        _in_variable_names={'_from': 'from', '_self': 'self',
            '_import': 'import'},
        _out_variable_name="return")
    def f(self, _from, _self, _import):
        return '1234'

class MultipleReturnService(ServiceBase):
    @rpc(String, _returns=(String, String, String))
    def multi(self, s):
        return s, 'a', 'b'

class Test(unittest.TestCase):
    '''Most of the service tests are performed through the interop tests.'''

    def _set_up(self):
        self.app = Application([TestService], rpclib.interface.wsdl.Wsdl11,
                                        rpclib.protocol.soap.Soap11, tns='tns')
        self.srv = TestService()
        self.app.interface.build_interface_document('URL')
        self._wsdl = self.app.interface.get_interface_document()
        self.wsdl = etree.fromstring(self._wsdl)

    def test_portypes(self):
        self._set_up()

        porttype = self.wsdl.find('{http://schemas.xmlsoap.org/wsdl/}portType')
        self.assertEquals(
            len(self.srv.public_methods), len(porttype.getchildren()))

    def test_override_param_names(self):
        self._set_up()

        for n in ['self', 'import', 'return', 'from']:
            self.assertTrue(n in self._wsdl, '"%s" not in self._wsdl' % n)

    def test_multiple_return(self):
        app = Application([MultipleReturnService], rpclib.interface.wsdl.Wsdl11,
                                        rpclib.protocol.soap.Soap11, tns='tns')
        app.interface.build_interface_document('url')
        srv = MultipleReturnService()
        message = srv.public_methods.values()[0].out_message()

        self.assertEquals(len(message._type_info), 3)

        sent_xml = etree.Element('test')
        message.to_parent_element( ('a','b','c'), srv.get_tns(), sent_xml )
        sent_xml = sent_xml[0]

        print etree.tostring(sent_xml, pretty_print=True)
        response_data = message.from_xml(sent_xml)

        self.assertEquals(len(response_data), 3)
        self.assertEqual(response_data[0], 'a')
        self.assertEqual(response_data[1], 'b')
        self.assertEqual(response_data[2], 'c')

    def test_multiple_ns(self):
        svc = Application([MultipleNamespaceService],rpclib.interface.wsdl.Wsdl11,
                                         rpclib.protocol.soap.Soap11,tns='tns')
        svc.interface.get_interface_document()

if __name__ == '__main__':
    unittest.main()




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""A soap client that uses http (urllib2) as transport"""

import urllib2

from rpclib.client import Service
from rpclib.client import ClientBase
from rpclib.client import RemoteProcedureBase

import rpclib.protocol.soap

class _RemoteProcedure(RemoteProcedureBase):
    def __call__(self, *args, **kwargs):
        self.get_out_object(args, kwargs) # sets self.ctx.out_object
        self.get_out_string()

        out_string = ''.join(self.ctx.out_string)
        request = urllib2.Request(self.url, out_string)
        code = 200
        try:
            response = urllib2.urlopen(request)
            self.ctx.in_string = response.read()

        except urllib2.HTTPError,e:
            code=e.code
            self.ctx.in_string = e.read()

        self.get_in_object()

        if not (self.ctx.in_error is None):
            raise self.ctx.in_error
        elif code >= 500:
            raise self.ctx.in_object
        else:
            return self.ctx.in_object

class Client(ClientBase):
    def __init__(self, url, app):
        super(Client, self).__init__(url, app)

        # FIXME: this four-line block should be explained...
        if isinstance(app.in_protocol, rpclib.protocol.soap._base._Soap11):
            app.in_protocol.in_wrapper = rpclib.protocol.soap._base._Soap11.OUT_WRAPPER
        if isinstance(app.out_protocol,rpclib.protocol.soap._base._Soap11):
            app.out_protocol.out_wrapper= rpclib.protocol.soap._base._Soap11.NO_WRAPPER

        self.service = Service(_RemoteProcedure, url, app)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

from collections import deque

from rpclib.const.xml_ns import DEFAULT_NS
from rpclib.util.oset import oset

class TransportContext(object):
    """Generic object that holds transport-specific context information"""
    def __init__(self, type=None):
        self.type=type

class EventContext(object):
    """Generic object that holds event-specific context information"""
    def __init__(self, event_id=None):
        self.event_id=event_id

class MethodContext(object):
    frozen = False

    @property
    def method_name(self):
        if self.descriptor is None:
            return None
        else:
            return self.descriptor.name

    def __init__(self, app):
        self.app = app

        self.udc = None  # the user defined context. use it to your liking.
        self.transport = TransportContext() # the transport-specific context.
        self.event = EventContext() # Event-specific context

        self.method_request_string = None

        # the following are set based on the value of the method_name.
        self.service_class = None  # the class the method belongs to
        self.descriptor = None     # its descriptor

        self.in_string = None      # incoming bytestream (can be any kind of
                                   #     iterable that contains strings)
        self.in_document = None    # parsed document
        self.in_error = None       # native python error object (probably a
                                   #     child of Exception)
        self.in_header_doc = None  # incoming header document of the request.
        self.in_body_doc = None    # incoming body document of the request.
        self.in_header = None      # native incoming header

        # in the request (i.e. server) case, this contains the function
        # arguments for the function in the service definition class.
        # in the response (i.e. client) case, this contains the object returned
        # by the remote procedure call.
        self.in_object = None

        # in the response (i.e. server) case, this is the native python object
        # returned by the function in the service definition class.
        # in the response (i.e. client) case, this contains the function
        # arguments passed to the function call wrapper.
        self.out_object = None

        self.out_header = None      # native python object set by the function
                                    # in the service definition class
        self.out_error = None       # native exception thrown by the function
                                    # in the service definition class
        self.out_body_doc = None    # serialized body object
        self.out_header_doc = None  # serialized header object
        self.out_document = None    # body and header wrapped in the outgoing
                                    # envelope
        self.out_string = None      # outgoing bytestream (can be any kind of
                                    # iterable that contains strings)

        self.frozen = True  # when this is set, no new attribute can be added
                            # to the class instance.

    def __setattr__(self, k, v):
        if self.frozen == False or k in self.__dict__:
            object.__setattr__(self, k, v)
        else:
            raise ValueError("use the udc member for storing arbitrary data "
                             "in the method context")

    def __repr__(self):
        retval = deque()
        for k, v in self.__dict__.items():
            if isinstance(v, dict):
                ret = deque(['{'])
                items = v.items()
                items.sort()
                for k2, v2 in items:
                    ret.append('\t\t%r: %r,' % (k2, v2))
                ret.append('\t}')
                ret = '\n'.join(ret)
                retval.append("\n\t%s=%s" % (k, ret))
            else:
                retval.append("\n\t%s=%r" % (k, v))

        retval.append('\n)')

        return ''.join((self.__class__.__name__, '(', ', '.join(retval), ')'))


class MethodDescriptor(object):
    '''This class represents the method signature of a soap method,
    and is returned by the rpc decorator.
    '''

    def __init__(self, function, in_message, out_message, doc,
                 is_callback=False, is_async=False, mtom=False, in_header=None,
                 out_header=None, faults=(),
                 port_type=None, no_ctx=False):

        self.function = function
        self.in_message = in_message
        self.out_message = out_message
        self.doc = doc
        self.is_callback = is_callback
        self.is_async = is_async
        self.mtom = mtom
        self.in_header = in_header
        self.out_header = out_header
        self.faults = faults
        self.port_type = port_type
        self.no_ctx = no_ctx

    @property
    def name(self):
        return self.in_message.get_type_name()

    @property
    def key(self):
        assert not (self.in_message.get_namespace() is DEFAULT_NS)

        return '{%s}%s' % (
            self.in_message.get_namespace(), self.in_message.get_type_name())

class EventManager(object):
    def __init__(self, parent, handlers={}):
        self.parent = parent
        self.handlers = dict(handlers)

    def add_listener(self, event_name, handler):
        handlers = self.handlers.get(event_name, oset())
        handlers.add(handler)
        self.handlers[event_name] = handlers

    def fire_event(self, event_name, ctx):
        handlers = self.handlers.get(event_name, oset())
        for handler in handlers:
            handler(ctx)




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from rpclib._base import MethodDescriptor
from rpclib.model.complex import ComplexModel
from rpclib.model.complex import TypeInfo
from rpclib.const.xml_ns import DEFAULT_NS

def _produce_input_message(f, params, _in_message_name, _in_variable_names, no_ctx):
    if no_ctx is True:
        arg_start=0
    else:
        arg_start=1

    argcount = f.func_code.co_argcount
    param_names = f.func_code.co_varnames[arg_start:argcount]

    in_params = TypeInfo()

    try:
        for i in range(len(param_names)):
            e0 = _in_variable_names.get(param_names[i], param_names[i])
            e1 = params[i]

            in_params[e0] = e1

    except IndexError, e:
        raise Exception("%r function's and its decorator's parameter numbers "
                        "mismatch." % f.func_name)

    ns = DEFAULT_NS
    if _in_message_name.startswith("{"):
        ns = _in_message_name[1:].partition("}")[0]

    message=ComplexModel.produce(type_name=_in_message_name, namespace=ns,
                                            members=in_params)
    message.__namespace__ = ns

    return message

def _validate_body_style(kparams):
    _body_style = kparams.get('_body_style')
    _soap_body_style = kparams.get('_soap_body_style')

    if _body_style is None:
        _body_style = 'wrapped'
    elif not (_body_style in ('wrapped', 'bare')):
        raise ValueError("body_style must be one of ('wrapped', 'bare')")
    elif _soap_body_style == 'document':
        _body_style = 'bare'
    elif _soap_body_style == 'rpc':
        _body_style = 'wrapped'
    else:
        raise ValueError("soap_body_style must be one of ('rpc', 'document')")
    assert _body_style in ('wrapped','bare')

    return _body_style

def _produce_output_message(f, func_name, kparams):
    """Generate an output message for "rpc"-style API methods.

    This message is a wrapper to the declared return type.
    """

    _returns = kparams.get('_returns')
    _body_style = _validate_body_style(kparams)

    _out_message_name = kparams.get('_out_message', '%sResponse' % func_name)
    out_params = TypeInfo()

    if _returns and _body_style == 'wrapped':
        if isinstance(_returns, (list, tuple)):
            default_names = ['%sResult%d' % (func_name, i) for i in
                                                           range(len(_returns))]

            _out_variable_names = kparams.get('_out_variable_names',
                                                                  default_names)

            assert (len(_returns) == len(_out_variable_names))

            var_pair = zip(_out_variable_names,_returns)
            out_params = TypeInfo(var_pair)

        else:
            _out_variable_name = kparams.get('_out_variable_name',
                                                       '%sResult' % func_name)

            out_params[_out_variable_name] = _returns

    ns = DEFAULT_NS
    if _out_message_name.startswith("{"):
        ns = _out_message_name[1:].partition("}")[0]

    if _body_style == 'wrapped':
        message = ComplexModel.produce(type_name=_out_message_name,
                                        namespace=ns,
                                        members=out_params)
        message.__namespace__ = ns # FIXME: is this necessary?

    else:
        message = ComplexModel.alias(_out_message_name, ns, _returns)

    return message

def srpc(*params, **kparams):
    kparams["_no_ctx"] = True
    return rpc(*params, **kparams)

def rpc(*params, **kparams):
    '''Method decorator to flag a method as a rpc-style operation.

    This is a method decorator to flag a method as a remote procedure call.  It
    will behave like a normal python method on a class, and will only behave
    differently when the keyword '_method_descriptor' is passed in, returning a
    'MethodDescriptor' object.  This decorator does none of the rpc
    serialization, only flags a method as a remotely callable procedure. This
    decorator should only be used on member methods of an instance of
    rpclib.service.ServiceBase.
    '''

    def explain(f):
        def explain_method(*args, **kwargs):
            retval = None
            function_name = kwargs['_default_function_name']

            _is_callback = kparams.get('_is_callback', False)
            _is_async = kparams.get('_is_async', False)
            _mtom = kparams.get('_mtom', False)
            _in_header = kparams.get('_in_header', None)
            _out_header = kparams.get('_out_header', None)
            _port_type = kparams.get('_soap_port_type', None)
            _no_ctx = kparams.get('_no_ctx', False)
            _faults = kparams.get('_faults', [])

            _in_message_name = kparams.get('_in_message_name', function_name)
            _in_variable_names = kparams.get('_in_variable_names', {})
            in_message = _produce_input_message(f, params, _in_message_name,
                                                    _in_variable_names, _no_ctx)

            out_message = _produce_output_message(f, function_name, kparams)

            doc = getattr(f, '__doc__')

            retval = MethodDescriptor(f,
                    in_message, out_message, doc, _is_callback, _is_async,
                    _mtom, _in_header, _out_header, _faults,
                    port_type=_port_type, no_ctx=_no_ctx)

            return retval

        explain_method.__doc__ = f.__doc__
        explain_method._is_rpc = True

        return explain_method
    return explain




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""A server that uses ZeroMQ (zmq.REP) as transport"""

import zmq

from rpclib._base import MethodContext
from rpclib.server import ServerBase

context = zmq.Context()

class ZmqMethodContext(MethodContext):
    def __init__(self, app):
        MethodContext.__init__(self, app)
        self.transport.type = 'zmq'

class Server(ServerBase):
    transport = 'http://rfc.zeromq.org/'

    def __init__(self, app, app_url, wsdl_url=None):
        ServerBase.__init__(self, app)

        self.app_url = app_url
        self.wsdl_url = wsdl_url

        self.soap_socket = context.socket(zmq.REP)
        self.soap_socket.bind(app_url)

    def __handle_wsdl_request(self):
        return self.app.get_interface_document(self.url)

    def serve_forever(self):
        while True:
            ctx = ZmqMethodContext(self.app)
            ctx.in_string = self.soap_socket.recv()

            self.get_in_object(ctx)

            if ctx.in_error:
                ctx.out_object = ctx.in_error
            else:
                self.get_out_object(ctx)
                if ctx.out_error:
                    ctx.out_object = ctx.out_error

            self.get_out_string(ctx)
            self.soap_socket.send(''.join(ctx.out_string))




#
# rpclib - Copyright (C) Rpclib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)

from lxml import etree

from rpclib.model import ModelBase
from rpclib.model import nillable_dict

from rpclib.util.odict import odict as TypeInfo
from rpclib.const import xml_ns as namespace

class XMLAttribute(ModelBase):
    """Items which are marshalled as attributes of the parent element."""

    def __init__(self, typ, use=None):
        self._typ = typ
        self._use = use

    def marshall(self, name, value, parent_elt):
        if value is not None:
            parent_elt.set(name, value)

    def describe(self, name, element):
        element.set('name', name)
        element.set('type', self._typ)
        if self._use:
            element.set('use', self._use)

class XMLAttributeRef(XMLAttribute):
    """Reference to stock XML attribute."""

    def __init__(self, ref, use=None):
        self._ref = ref
        self._use = use

    def describe(self, name, element):
        element.set('ref', self._ref)
        if self._use:
            element.set('use', self._use)

class SelfReference(object):
    pass

class ComplexModelMeta(type(ModelBase)):
    '''
    This is the metaclass that populates ComplexModel instances with
    the appropriate datatypes for (de)serialization.
    '''

    def __new__(cls, cls_name, cls_bases, cls_dict):
        '''This initializes the class, and registers attributes for
        serialization.
        '''

        type_name = cls_dict.get("__type_name__", None)
        if type_name is None:
            cls_dict["__type_name__"] = cls_name

        # get base class (if exists) and enforce single inheritance
        extends = cls_dict.get("__extends__", None)

        if extends is None:
            for b in cls_bases:
                base_types = getattr(b, "_type_info", None)

                if not (base_types is None):
                    if not (extends is None or cls_dict["__extends__"] is b):
                        raise Exception("WSDL 1.1 does not support multiple "
                                        "inheritance")

                    try:
                        if len(base_types) > 0 and issubclass(b, ModelBase):
                            cls_dict["__extends__"] = extends = b
                    except:
                        logger.error(repr(extends))
                        raise

        # populate soap members
        if not ('_type_info' in cls_dict):
            cls_dict['_type_info'] = _type_info = TypeInfo()

            for k, v in cls_dict.items():
                if not k.startswith('__'):
                    attr = isinstance(v, XMLAttribute)
                    try:
                        subc = issubclass(v, ModelBase)
                    except:
                        subc = False

                    if subc:
                        _type_info[k] = v
                        if issubclass(v, Array) and v.serializer is None:
                            raise Exception("%s.%s is an array of what?" %
                                                                  (cls_name, k))
                    elif attr:
                        _type_info[k] = v

        else:
            _type_info = cls_dict['_type_info']
            if not isinstance(_type_info, TypeInfo):
                cls_dict['_type_info'] = TypeInfo(_type_info)

        return type(ModelBase).__new__(cls, cls_name, cls_bases, cls_dict)

    def __init__(self, cls_name, cls_bases, cls_dict):
        for k in cls_dict:
            if cls_dict[k] is SelfReference:
                cls_dict[k] = self
                self._type_info[k] = self

        type(ModelBase).__init__(self, cls_name, cls_bases, cls_dict)

class ComplexModelBase(ModelBase):
    """If you want to make a better class type, this is what you should inherit
    from.
    """

    def __init__(self, **kwargs):
        super(ComplexModelBase, self).__init__()

        self.__reset_members(self.__class__, kwargs)

    def __reset_members(self, cls, kwargs):
        extends = getattr(cls, "__extends__", None)
        if not (extends is None):
            self.__reset_members(extends, kwargs)

        for k in cls._type_info.keys():
            setattr(self, k, kwargs.get(k, None))

    def __len__(self):
        return len(self._type_info)

    def __getitem__(self, i):
        return getattr(self, self._type_info.keys()[i], None)

    @classmethod
    def get_serialization_instance(cls, value):
        # if the instance is a list, convert it to a cls instance.
        # this is only useful when deserializing descriptor.in_message as it's
        # the only time when the member order is not arbitrary (as the members
        # are declared and passed around as sequences of arguments, unlike
        # dictionaries in a regular class definition).
        if isinstance(value, list) or isinstance(value, tuple):
            assert len(value) <= len(cls._type_info)

            inst = cls()

            keys = cls._type_info.keys()
            for i in range(len(value)):
                setattr(inst, keys[i], value[i])

        elif isinstance(value, dict):
            inst = cls()

            for k in cls._type_info:
                setattr(inst, k, value.get(k, None))

        else:
            inst = value

        return inst

    @classmethod
    def get_deserialization_instance(cls):
        return cls()

    @classmethod
    def get_members_pairs(cls, inst):
        parent_cls = getattr(cls, '__extends__', None)
        if not (parent_cls is None):
            for r in parent_cls.get_members_pairs(inst, parent):
                yield r

        for k, v in cls._type_info.items():
            mo = v.Attributes.max_occurs
            subvalue = getattr(inst, k, None)

            if mo == 'unbounded' or mo > 1:
                if subvalue != None:
                    yield (k, [v.to_string(sv) for sv in subvalue])

            else:
                yield k, v.to_string(subvalue)

    @classmethod
    @nillable_dict
    def to_dict(cls, value):
        inst = cls.get_serialization_instance(value)

        return dict(cls.get_members_pairs(inst))

    @staticmethod
    def get_flat_type_info(clz, retval={}):
        parent = getattr(clz, '__extends__', None)
        if parent != None:
            clz.get_flat_type_info(parent, retval)

        retval.update(clz._type_info)

        return retval

    @classmethod
    @nillable_dict
    def from_dict(cls, in_dict):
        inst = cls.get_deserialization_instance()
        flat_type_info = ComplexModelBase.get_flat_type_info(cls)

        # initialize instance
        for k in flat_type_info:
            setattr(inst, k, None)

        # initialize instance
        for k, v in in_dict.items():
            member = flat_type_info.get(k, None)
            if member is None:
                continue

            mo = member.Attributes.max_occurs
            logger.debug("%r, %r: %r, %r" % (member, k, v, mo))
            if mo == 'unbounded' or mo > 1:
                value = getattr(inst, k, None)
                if value is None:
                    value = []

                for v2 in v:
                    value.append(member.from_string(v2))

                setattr(inst, k, value)

            else:
                v,  = v
                setattr(inst, k, member.from_string(v))

        return inst

    @staticmethod
    def resolve_namespace(cls, default_ns):
        if getattr(cls, '__extends__', None) != None:
            cls.__extends__.resolve_namespace(cls.__extends__, default_ns)

        ModelBase.resolve_namespace(cls, default_ns)

        for k, v in cls._type_info.items():
            if v.__type_name__ is ModelBase.Empty:
                v.__namespace__ = cls.get_namespace()
                v.__type_name__ = "%s_%sType" % (cls.get_type_name(), k)

            if v != cls:
                v.resolve_namespace(v, default_ns)

    @staticmethod
    def produce(namespace, type_name, members):
        """Lets you create a class programmatically."""

        cls_dict = {}

        cls_dict['__namespace__'] = namespace
        cls_dict['__type_name__'] = type_name
        cls_dict['_type_info'] = TypeInfo(members)

        return ComplexModelMeta(type_name, (ComplexModel, ), cls_dict)

    @staticmethod
    def alias(type_name, namespace, target):
        """Return an alias class for the given target class.

        This function is a variation on 'ComplexModel.produce'. The alias will
        borrow the target's _type_info.
        """

        cls_dict = {}

        cls_dict['__namespace__'] = namespace
        cls_dict['__type_name__'] = type_name
        cls_dict['_type_info'] = getattr(target, '_type_info', ())
        cls_dict['_target'] = target

        return ComplexModelMeta(type_name, (ClassAlias, ), cls_dict)

class ComplexModel(ComplexModelBase):
    """
    The general complexType factory. The __call__ method of this class will
    return instances, contrary to primivites where the same call will result in
    customized duplicates of the original class definition.
    Those who'd like to customize the class should use the customize method.
    (see rpclib.model.base.ModelBase)
    """

    __metaclass__ = ComplexModelMeta

class Array(ComplexModel):
    def __new__(cls, serializer, **kwargs):
        retval = cls.customize(**kwargs)

        # hack to default to unbounded arrays when the user didn't specify
        # max_occurs. We should find a better way.
        if serializer.Attributes.max_occurs == 1:
            serializer = serializer.customize(max_occurs='unbounded')

        if serializer.get_type_name() is ModelBase.Empty:
            member_name = serializer.__base_type__.get_type_name()
            if cls.__type_name__ is None:
                cls.__type_name__ = ModelBase.Empty # to be resolved later

        else:
            member_name = serializer.get_type_name()
            if cls.__type_name__ is None:
                cls.__type_name__ = '%sArray' % serializer.get_type_name()

        retval.__type_name__ = '%sArray' % member_name
        retval._type_info = {member_name: serializer}

        return retval

    # the array belongs to its child's namespace, it doesn't have its own
    # namespace.
    @staticmethod
    def resolve_namespace(cls, default_ns):
        (serializer, ) = cls._type_info.values()

        serializer.resolve_namespace(serializer, default_ns)

        if cls.__namespace__ is None:
            cls.__namespace__ = serializer.get_namespace()

        if cls.__namespace__ in namespace.const_prefmap:
            cls.__namespace__ = default_ns

        ComplexModel.resolve_namespace(cls, default_ns)

    @classmethod
    def get_serialization_instance(cls, value):
        inst = ComplexModel.__new__(Array)

        (member_name, ) = cls._type_info.keys()
        setattr(inst, member_name, value)

        return inst

class Iterable(Array):
    pass

class Alias(ComplexModel):
    """New type_name, same _type_info."""
